package jp.gr.java_conf.coskx.gles2sample09r;

import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.opengl.Matrix;
import android.util.Log;
import android.view.Display;
import android.view.Surface;

/**
 * Created by tommy on 2017/10/28.
 */


//★★★　縦位置(Portrait)アプリケーションの場合　ここから　★★★
//androidの上下のバーがPortraitで固定しているだけで，タブレットをどの向きにしても使える
public class RotationSensor {
    private Context mContext;
    private Display mDisplay;
    private SensorManager mSensorManager = null;
    private SensorEventListener mSensorEventListener = null;

    float[] rotmatrix= new float[16];

    public RotationSensor(Context context, Display display) {
        mContext = context;
        mDisplay =display;
        mSensorManager = (SensorManager) mContext.getSystemService( Context.SENSOR_SERVICE );

        mSensorEventListener = new SensorEventListener()
        {
            float[] rMat = new float[16];

            public void onSensorChanged (SensorEvent event) {
                if ( event.sensor.getType() == Sensor.TYPE_ROTATION_VECTOR ) {
                    SensorManager.getRotationMatrixFromVector(rMat, event.values);
                    //ｒMatはportraitの時の回転行列
                    //そのまま 端末を立ててportrait　rMatとrotmatrixは同じもの
                    SensorManager.remapCoordinateSystem(rMat, SensorManager.AXIS_X, SensorManager.AXIS_Y, rotmatrix );
                }
            }
            public void onAccuracyChanged (Sensor sensor, int accuracy) {}
        };
    }

    //成功した場合は0を返す
    //ROTATION_VECTOR sensor が無かった場合は-1を返す
    public int setSensorsOn() {
        Matrix.setIdentityM(rotmatrix, 0);
        Sensor sensor;
        sensor= mSensorManager.getDefaultSensor( Sensor.TYPE_ROTATION_VECTOR );
        if (sensor==null) return -1;
        mSensorManager.registerListener(
                mSensorEventListener,
                mSensorManager.getDefaultSensor( Sensor.TYPE_ROTATION_VECTOR ),
                SensorManager.SENSOR_DELAY_UI );
        return 0;
    }

    public void setSensorsOff() {
        //センサーリスナーを解除
        mSensorManager.unregisterListener( mSensorEventListener );
    }

    public float[] getRotMatrix() {
        float[] mat=rotmatrix.clone();
        return mat;
    }

}
//★★★　縦位置(Portrait)アプリケーションの場合　ここまで　★★★


/*
//★★★　横位置(landscape)アプリケーションの場合　ここから　★★★
//androidの上下のバーがlansdcapeで固定しているだけで，タブレットをどの向きにしても使える
public class RotationSensor {
    private Context mContext;
    private Display mDisplay;
    private SensorManager mSensorManager = null;
    private SensorEventListener mSensorEventListener = null;

    float[] rotmatrix= new float[16];

    public RotationSensor(Context context, Display display) {
        mContext = context;
        mDisplay = display;
        mSensorManager = (SensorManager) mContext.getSystemService( Context.SENSOR_SERVICE );

        mSensorEventListener = new SensorEventListener()
        {
            float[] rMat = new float[16];

            public void onSensorChanged (SensorEvent event) {
                if ( event.sensor.getType() == Sensor.TYPE_ROTATION_VECTOR) {
                    SensorManager.getRotationMatrixFromVector(rMat, event.values);
                    //ｒMatはportraitの時の回転行列
                    // ワールド座標とデバイス座標のマッピングを変換する 端末を立ててlandscape（portrait時の上部を左に倒した）
                    SensorManager.remapCoordinateSystem(rMat, SensorManager.AXIS_Y, SensorManager.AXIS_MINUS_X, rotmatrix );
                }
            }
            public void onAccuracyChanged (Sensor sensor, int accuracy) {}
        };
    }

    //成功した場合は0を返す
    //ROTATION_VECTOR sensor が無かった場合は-1を返す
    public int setSensorsOn() {
        Matrix.setIdentityM(rotmatrix, 0);
        Sensor sensor;
        sensor= mSensorManager.getDefaultSensor( Sensor.TYPE_ROTATION_VECTOR );
        if (sensor==null) return -1;
        mSensorManager.registerListener(
                mSensorEventListener,
                mSensorManager.getDefaultSensor( Sensor.TYPE_ROTATION_VECTOR ),
                SensorManager.SENSOR_DELAY_UI );
        return 0;
    }

    public void setSensorsOff() {
        //センサーリスナーを解除
        mSensorManager.unregisterListener( mSensorEventListener );
    }

    public float[] getRotMatrix() {
        float[] mat=rotmatrix.clone();
        return mat;
    }

}
//★★★　横位置(landscape)アプリケーションの場合　ここまで　★★★
*/

/*
//★★★　縦位置(portrait)・横位置(landscape)両様アプリケーションの場合　ここから　★★★
//画面の面内自動回転を有効にした場合，画面の面内自動回転に追随する
//androidの上下のバーの位置が，姿勢によって移動するが，タブレットをどの向きにしても使える
public class RotationSensor {
    private Context mContext;
    private Display mDisplay;
    private SensorManager mSensorManager = null;
    private SensorEventListener mSensorEventListener = null;

    float[] rotmatrix= new float[16];

    public RotationSensor(Context context, Display display) {
        mContext = context;
        mDisplay =display;
        mSensorManager = (SensorManager) mContext.getSystemService( Context.SENSOR_SERVICE );

        mSensorEventListener = new SensorEventListener()
        {
            float[] rMat = new float[16];
            int surfacerotation=-1; //画面の面内の回転

            public void onSensorChanged (SensorEvent event) {
                if (event.sensor.getType() == Sensor.TYPE_ROTATION_VECTOR) {
                    // calculate th rotation matrix
                    SensorManager.getRotationMatrixFromVector(rMat, event.values);
                    //ｒMatはportraitの時の回転行列
                    surfacerotation = getRotationValue();
                    switch (surfacerotation) {
                        case 0:
                            //そのまま 端末を立ててportrait
                            SensorManager.remapCoordinateSystem(rMat, SensorManager.AXIS_X, SensorManager.AXIS_Y, rotmatrix );
                            break;
                        case 90:
                            // ワールド座標とデバイス座標のマッピングを変換する 端末を立ててlandscape（portrait時の上部を左に倒した）
                            SensorManager.remapCoordinateSystem(rMat, SensorManager.AXIS_Y, SensorManager.AXIS_MINUS_X, rotmatrix );
                            break;
                        case 180:
                            // ワールド座標とデバイス座標のマッピングを変換する 端末を立てて上下逆のportrait
                            SensorManager.remapCoordinateSystem(rMat, SensorManager.AXIS_MINUS_X, SensorManager.AXIS_MINUS_Y, rotmatrix );
                            break;
                        case 270:
                            // ワールド座標とデバイス座標のマッピングを変換する 端末を立てて上下逆のlandscape（portrait時の上部を右に倒した）
                            SensorManager.remapCoordinateSystem(rMat, SensorManager.AXIS_MINUS_Y, SensorManager.AXIS_X, rotmatrix );
                            break;
                    }
                }
            }

            public void onAccuracyChanged (Sensor sensor, int accuracy) {}
        };
    }

    //成功した場合は0を返す
    //ROTATION_VECTOR sensor が無かった場合は-1を返す
    public int setSensorsOn() {
        Matrix.setIdentityM(rotmatrix, 0);
        Sensor sensor;
        sensor= mSensorManager.getDefaultSensor( Sensor.TYPE_ROTATION_VECTOR );
        if (sensor==null) return -1;
        mSensorManager.registerListener(
                mSensorEventListener,
                mSensorManager.getDefaultSensor( Sensor.TYPE_ROTATION_VECTOR ),
                SensorManager.SENSOR_DELAY_UI );
        return 0;
    }

    public void setSensorsOff() {
        //センサーリスナーを解除
        mSensorManager.unregisterListener( mSensorEventListener );
    }

    public float[] getRotMatrix() {
        float[] mat=rotmatrix.clone();
        return mat;
    }

    //端末の画面の向きを取得する。
    //返す値 画面の回転角度
    public int getRotationValue() {
        int val = -1;
        //==== 画面向き取得 ====//
        int rotation = mDisplay.getRotation();
        switch(rotation) {
            case Surface.ROTATION_0:
                //端末を立ててportrait　背面カメラでportrait（縦位置）写真をとる状態
                val = 0;
                break;
            case Surface.ROTATION_90:
                //端末を立ててportrait状態から左に90度倒したlandscape　背面カメラでlandscape（横位置）写真をとる状態
                val = 90;
                break;
            case Surface.ROTATION_180:
                //端末を立ててportrait状態から180度倒した逆portrait
                val = 180;
                break;
            case Surface.ROTATION_270:
                //端末を立ててportrait状態から右に90度倒した逆landscape
                val = 270;
                break;
        }
        return val;
    }
}
//★★★　縦位置(portrait)・横位置(landscape)両様アプリケーションの場合　ここまで★
*/
