package jp.gr.java_conf.coskx.gles2sample09r2;

import android.content.Context;
import android.graphics.Color;
import android.opengl.GLES20;
import android.opengl.GLSurfaceView;
import android.opengl.Matrix;

import java.nio.FloatBuffer;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;


/**
 * Created by tommy on 2015/06/18.
 */
public class GLRenderer implements GLSurfaceView.Renderer {
    //システム
    private final Context mContext;
    private boolean validProgram=false; //シェーダプログラムが有効
    private RotationSensor myRotationSensor=null; //加速度方位GPSセンサ

    private float aspect;//アスペクト比
    private float viewingangle = 45f; //視野角（画面長手方向に対応する視野角[deg]）

    //視点変更テスト変数
    private float alph=0f,beta=0f;

    //光源の座標　x,y,z
    private  float[] LightPos={0f,1.5f,3f,1f};//x,y,z,1

    //変換マトリックス
    private  float[] pMatrix=new float[16]; //プロジェクション変換マトリックス
    private  float[] mMatrix=new float[16]; //モデル変換マトリックス
    private  float[] cMatrix=new float[16]; //カメラビュー変換マトリックス

    //モデル座標系の原点
    private  float[] origin= {0f,0f,0f,1f};

    private Axes MyAxes= new Axes();  //原点周囲の軸表示とためのオブジェクトを作成
    private WireSphere MyWireSphere = new WireSphere(40,20); //原点に，半径１の球体オブジェクト（40スライス，20スタック）を作成
    private TexCube MyTexCube = new TexCube(); //原点に，外接球半径１の立方体オブジェクトを作成
    private StringTexture North;
    private StringTexture South;
    private StringTexture East;
    private StringTexture West;
    private StringTexture Upper;
    private StringTexture Lower;

    //シェーダのattribute属性の変数に値を設定していないと暴走するのでそのための準備
    private static float[] DummyFloat= new float[1];
    private static final FloatBuffer DummyBuffer=BufferUtil.makeFloatBuffer(DummyFloat);

    GLRenderer(final Context context) {
        mContext = context;
    }

    //サーフェイス生成時に呼ばれる
    @Override
    public void onSurfaceCreated(GL10 gl10,EGLConfig eglConfig) {
        //プログラムの生成
        validProgram = GLES.makeProgram();

        //頂点配列の有効化
        GLES20.glEnableVertexAttribArray(GLES.positionHandle);
        GLES20.glEnableVertexAttribArray(GLES.normalHandle);
        GLES20.glEnableVertexAttribArray(GLES.texcoordHandle);

        //デプスバッファの有効化
        GLES20.glEnable(GLES20.GL_DEPTH_TEST);

        // カリングの有効化
        GLES20.glEnable(GLES20.GL_CULL_FACE); //裏面を表示しないチェックを行う

        // 裏面を描画しない
        GLES20.glFrontFace(GLES20.GL_CCW); //表面のvertexのindex番号はCCWで登録
        GLES20.glCullFace(GLES20.GL_BACK); //裏面は表示しない

        //光源色の指定 (r, g, b,a)
        GLES20.glUniform4f(GLES.lightAmbientHandle, 0.15f, 0.15f, 0.15f, 1.0f); //周辺光
        GLES20.glUniform4f(GLES.lightDiffuseHandle, 0.5f, 0.5f, 0.5f, 1.0f); //乱反射光
        GLES20.glUniform4f(GLES.lightSpecularHandle, 0.9f, 0.9f, 0.9f, 1.0f); //鏡面反射光

        //背景色の設定
        GLES20.glClearColor(0f, 0f, 0.2f, 1.0f);

        //テクスチャの有効化
        GLES20.glEnable(GLES20.GL_TEXTURE_2D);

        // 背景とのブレンド方法を設定します。
        GLES20.glEnable(GLES20.GL_BLEND);
        GLES20.glBlendFunc(GLES20.GL_SRC_ALPHA, GLES20.GL_ONE_MINUS_SRC_ALPHA);    // 単純なアルファブレンド

        North = new StringTexture("North",20, Color.WHITE, Color.CYAN); //文字列テクスチャを作成
        South = new StringTexture("South",20, Color.WHITE, Color.RED); //文字列テクスチャを作成
        East = new StringTexture("East",20, Color.WHITE, Color.GREEN); //文字列テクスチャを作成
        West = new StringTexture("West",20, Color.WHITE, Color.MAGENTA); //文字列テクスチャを作成
        Upper = new StringTexture("Upper",20, Color.WHITE, Color.LTGRAY); //文字列テクスチャを作成
        Lower = new StringTexture("Lower",20, Color.WHITE, Color.DKGRAY); //文字列テクスチャを作成

    }

    public void setRotationSensor(RotationSensor s) {
        myRotationSensor=s;
    }

    //画面サイズ変更時に呼ばれる
    @Override
    public void onSurfaceChanged(GL10 gl10,int w,int h) {
        //ビューポート変換
        GLES20.glViewport(0,0,w,h);
        aspect=(float)w/(float)h;
    }

    //毎フレーム描画時に呼ばれる
    @Override
    public void onDrawFrame(GL10 glUnused) {
        if (!validProgram) return;
        //シェーダのattribute属性の変数に値を設定していないと暴走するのでここでセットしておく。この位置でないといけない
        GLES20.glVertexAttribPointer(GLES.positionHandle, 3, GLES20.GL_FLOAT, false, 0, DummyBuffer);
        GLES20.glVertexAttribPointer(GLES.normalHandle, 3, GLES20.GL_FLOAT, false, 0, DummyBuffer);
        GLES20.glVertexAttribPointer(GLES.texcoordHandle, 2, GLES20.GL_FLOAT, false, 0, DummyBuffer);

        GLES.disableTexture();  //テクスチャ機能を無効にする。（デフォルト）
        GLES.enableShading();   //シェーディング機能を有効にする。（デフォルト）

        //画面のクリア
        GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT |
                GLES20.GL_DEPTH_BUFFER_BIT);

        //プロジェクション変換（射影変換）--------------------------------------
        //透視変換（遠近感を作る）
        //カメラは原点に有り，z軸の負の方向を向いていて，上方向はy軸＋方向である。
        GLES.gluPerspective(pMatrix,
                viewingangle,  //Y方向の画角
                aspect, //アスペクト比
                .01f,   //ニアクリップ　　　z=-.1から
                120.0f);//ファークリップ　　Z=-10までの範囲を表示することになる
        GLES.setPMatrix(pMatrix);

        float[] c1Matrix=new float[16]; //カメラ視点変換マトリックス作成用
        float[] c2Matrix=new float[16]; //カメラ視点変換マトリックス作成用
        //カメラビュー変換（視野変換）-----------------------------------
        //portraitで使うアプリを想定
        //カメラ視点が原点になるような変換　カメラを下向き，長手方向奥側を2D画面の上にする
        Matrix.setLookAtM(c1Matrix, 0,
                0f,0f,0f,  //カメラの視点 x,y,z
                0.0f, 0.0f, -1.0f, //カメラの焦点
                0.0f, 1.0f, 0.0f);//カメラの上方向  画面長手方向前方
        c2Matrix = myRotationSensor.getRotMatrix();

        //次の1行を挿入すると，カメラ位置が，原点ではなく(6,6,6)になるため，6個の立方体が南西下方に俯瞰できる
        //Matrix.translateM(c2Matrix,0,-6f,-6f,-6f);

        //test
        //Matrix.rotateM(c2Matrix,0,90f,1f,0f,0f);
        //Matrix.translateM(c2Matrix,0,0f,0f,-6f);

        Matrix.multiplyMM(cMatrix, 0, c1Matrix, 0, c2Matrix, 0); //cMatrix = c1Matrix * c2Matrix

        //カメラビュー変換はこれで終わり。
        GLES.setCMatrix(cMatrix);

        //cMatrixをセットしてから光源位置をセット
        GLES.setLightPosition(LightPos);

        //座標軸，ワイヤフレーム球の描画
        GLES.disableShading(); //シェーディング機能は使わない
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.translateM(mMatrix, 0, 0.5f, -.1f, -.2f);
        Matrix.scaleM(mMatrix, 0, 0.1f, 0.1f, 0.1f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        //座標軸の描画本体
        //引数 r, g, b, a, shininess(1以上の値　大きな値ほど鋭くなる), linewidth
        //shininessは使用していない
        MyAxes.draw(1f, 1f, 1f, 1f, 10.f, 2f);//座標軸の描画本体

        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.scaleM(mMatrix, 0, 100f, 100f, 100f);
        Matrix.rotateM(mMatrix, 0, 90f, 1f, 0f,0f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        MyWireSphere.draw(.5f, 0f, 0f, 1f); //ワイアフレームの球

        GLES.enableShading(); //シェーディング機能を使う設定に戻す

        GLES.enableTexture();

        Matrix.setIdentityM(mMatrix, 0);
        Matrix.translateM(mMatrix, 0, 0f, 2.0f, 0f);
        Matrix.scaleM(mMatrix, 0, 0.4f, 0.4f, 0.4f);
        Matrix.rotateM(mMatrix, 0, 90f, 0f, 1f,0f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        North.setTexture();
        // r, g, b, a, shininess(1以上の値　大きな値ほど鋭くなる)
        MyTexCube.draw(1f, 1f, 1f, 1f, 20.f);

        Matrix.setIdentityM(mMatrix, 0);
        Matrix.translateM(mMatrix, 0, 0f, -2.0f, 0f);
        Matrix.scaleM(mMatrix, 0, 0.4f, 0.4f, 0.4f);
        Matrix.rotateM(mMatrix, 0, -90f, 0f, 1f,0f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        South.setTexture();
        // r, g, b, a, shininess(1以上の値　大きな値ほど鋭くなる)
        MyTexCube.draw(1f, 1f, 1f, 1f, 20.f);

        Matrix.setIdentityM(mMatrix, 0);
        Matrix.translateM(mMatrix, 0, 2.0f, 0f, 0f);
        Matrix.scaleM(mMatrix, 0, 0.4f, 0.4f, 0.4f);
        Matrix.rotateM(mMatrix, 0, 90f, 1f, 0f,0f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        East.setTexture();
        // r, g, b, a, shininess(1以上の値　大きな値ほど鋭くなる)
        MyTexCube.draw(1f, 1f, 1f, 1f, 20.f);

        Matrix.setIdentityM(mMatrix, 0);
        Matrix.translateM(mMatrix, 0, -2.0f, 0f, 0f);
        Matrix.scaleM(mMatrix, 0, 0.4f, 0.4f, 0.4f);
        Matrix.rotateM(mMatrix, 0, -90f, 1f, 0f,0f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        West.setTexture();
        // r, g, b, a, shininess(1以上の値　大きな値ほど鋭くなる)
        MyTexCube.draw(1f, 1f, 1f, 1f, 20.f);

        Matrix.setIdentityM(mMatrix, 0);
        Matrix.translateM(mMatrix, 0, 0f, 0f, 2.f);
        Matrix.scaleM(mMatrix, 0, 0.4f, 0.4f, 0.4f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        Upper.setTexture();
        // r, g, b, a, shininess(1以上の値　大きな値ほど鋭くなる)
        MyTexCube.draw(1f, 1f, 1f, 1f, 20.f);

        Matrix.setIdentityM(mMatrix, 0);
        Matrix.translateM(mMatrix, 0, 0f, 0f, -2f);
        Matrix.scaleM(mMatrix, 0, 0.4f, 0.4f, 0.4f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        Lower.setTexture();
        // r, g, b, a, shininess(1以上の値　大きな値ほど鋭くなる)
        MyTexCube.draw(1f, 1f, 1f, 1f, 20.f);

        GLES.disableTexture();
    }

}
