package jp.gr.java_conf.coskx.gles2sample09r2;

import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.opengl.Matrix;
import android.util.Log;
import android.view.Display;
import android.view.Surface;

/**
 * Created by tommy on 2017/10/28.
 */

/*
デバイスの姿勢取得をするために使用するセンサは次のどちらかを使う。
(1)ROTATION_VECTORセンサとGYROSCOPEセンサを用いる（動揺が少ない）
(2)MAGNETIC_FIELDセンサとACCELEROMETERセンサを用いる（遅れと動揺がある）

ROTATION_VECTORセンサはGYROSCOPEセンサを内包するため
GYROSCOPEセンサの利用の可否を検査し，(1)(2)のどちらで使うかを決めれば良い。

Androidの古い機種の場合，GYROSCOPEセンサ，ROTATION_VECTORセンサの利用の可否を正しく答えない場合がある。
そのため，ROTATION_VECTORセンサからの出力値の回転行列をチェックする，
回転行列は直交行列のはずなので，このことを使って検査する
行列が直交行列ならば，当該行列とその転置行列の積は単位行列になる。
01-19 13:16:46.882 18144-18144/jp.gr.java_conf.coskx.test_attitude I/RotationSensor: ROTATION_VECTOR orthogonal check = 1.3927309E-10
01-19 13:16:46.946 18144-18144/jp.gr.java_conf.coskx.test_attitude I/RotationSensor: ROTATION_VECTOR orthogonal check = 5.655729E-10
01-19 13:16:47.012 18144-18144/jp.gr.java_conf.coskx.test_attitude I/RotationSensor: ROTATION_VECTOR orthogonal check = 4.0576956E-10
01-19 13:16:47.082 18144-18144/jp.gr.java_conf.coskx.test_attitude I/RotationSensor: ROTATION_VECTOR orthogonal check = 6.767378E-11
01-19 13:16:47.147 18144-18144/jp.gr.java_conf.coskx.test_attitude I/RotationSensor: ROTATION_VECTOR orthogonal check = 9.7164665E-11
01-19 13:16:47.218 18144-18144/jp.gr.java_conf.coskx.test_attitude I/RotationSensor: ROTATION_VECTOR orthogonal check = 9.45697E-11
0.1以上になったら直交行列ではないと考える。
*/


//★★★　縦位置(Portrait)アプリケーションの場合　ここから　★★★
//androidの上下のバーがPortraitで固定しているだけで，タブレットをどの向きにしても使える
public class RotationSensor {
    private Context mContext;
    private Display mDisplay;
    private SensorManager mSensorManager = null;
    private SensorEventListener mSensorEventListener = null;
    private int sensormode=0; //1,2,3
    private int isReady=0; //センサ値の準備状況
    private boolean firsttime = true;

    float[] rotmatrix= new float[16];
    float[] rotmatrix0= new float[16];
    float[] imatrix= new float[16];
    private float[] fGyroscope = new float[3];
    private float[] fmagneticValues = new float[3];
    private float[] faccelerometerValues = new float[3];
    private int errorcount=0;
    private int noerrorcount=0;
    private int surfacerotation;
    private int x_axis, y_axis;

    public RotationSensor(Context context, Display display) {
        mContext = context;
        mDisplay =display;
        mSensorManager = (SensorManager) mContext.getSystemService( Context.SENSOR_SERVICE );

        mSensorEventListener = new SensorEventListener()
        {
            float[] rMat = new float[16];

            public void onSensorChanged (SensorEvent event) {
                switch( event.sensor.getType()) {
                    case Sensor.TYPE_ROTATION_VECTOR:
                        SensorManager.getRotationMatrixFromVector(rMat, event.values);
                        //ｒMatはportraitの時の回転行列
                        //そのまま 端末を立ててportrait　rMatとrotmatrixは同じもの
                        SensorManager.remapCoordinateSystem(rMat, x_axis, y_axis, rotmatrix0);
                        //Log.i("RotationSensor", "ROTATION_VECTOR =  (" + rotmatrix0[0] + ", "+ rotmatrix0[4] + ", "+ rotmatrix0[8] + "), ("
                        //        + rotmatrix0[1] + ", "+ rotmatrix0[5] + ", "+ rotmatrix0[9] + "), ("
                        //        + rotmatrix0[2] + ", "+ rotmatrix0[6] + ", "+ rotmatrix0[10] + ")");
                        if (noerrorcount<200) {
                            float errorvalue = checkOrthogonalMatrix(rotmatrix0);
                            Log.i("RotationSensor", "ROTATION_VECTOR orthogonal check = " + errorvalue);
                            if (0.1f<errorvalue) {
                                errorcount++;
                                if (50<errorcount) changeSensor();
                            } else {
                                noerrorcount++;
                            }
                        }
                        isReady |= 1;
                        break;
                    case Sensor.TYPE_GYROSCOPE:
                        fGyroscope = event.values.clone();
                        //Log.i("RotationSensor", "GYROSCOPE = " + fGyroscope[0] + ", "+ fGyroscope[1] + ", "+ fGyroscope[2]);
                        isReady |= 2;
                        break;
                    case Sensor.TYPE_MAGNETIC_FIELD:
                        fmagneticValues = event.values.clone();
                        isReady |= 4;
                        break;
                    case Sensor.TYPE_ACCELEROMETER:
                        faccelerometerValues = event.values.clone();
                        isReady |= 8;
                        break;
                }
                if (firsttime) {
                    if (sensormode==1) {
                        if ((isReady&1)==1) {
                            for (int i=0; i<3; i++) {
                                for (int j=0; j<3; j++) {
                                    rotmatrix[4*i+j]=rotmatrix0[4*i+j];
                                }
                            }
                            firsttime = false;
                        }
                    } else {
                        if ((isReady&0xc)==0xc) {
                            SensorManager.getRotationMatrix(rMat, imatrix, faccelerometerValues, fmagneticValues);
                            SensorManager.remapCoordinateSystem(rMat, x_axis, y_axis, rotmatrix);
                            getMovement(faccelerometerValues, fmagneticValues);
                            firsttime = false;
                        }
                    }
                } else {
                    if (sensormode==1) {
                        if ((isReady & 3) == 3) {
                            float gyr=getNorm(fGyroscope);
                            if (1f<gyr) gyr=1f;
                            else if (gyr<0.01f) gyr=.01f;
                            float weight=1f-gyr;
                            for (int i=0; i<3; i++) {
                                for (int j=0; j<3; j++) {
                                    rotmatrix[4*i+j]=weight*rotmatrix[4*i+j]+gyr*rotmatrix0[4*i+j];
                                }
                            }
                        }
                    } else if (sensormode==2) {
                        float weight1=getMovement(faccelerometerValues, fmagneticValues)/30f;
                        //Log.i("RotationSEnsor","mode3 weight1 = "+ weight1);
                        if (1f<weight1) weight1=1f;
                        else if (weight1<0.001f) weight1=0.001f;
                        float weight0=1f-weight1;
                        if ((isReady&0xc)==0xc) {
                            SensorManager.getRotationMatrix(rMat, imatrix, faccelerometerValues, fmagneticValues);
                            SensorManager.remapCoordinateSystem(rMat, x_axis, y_axis, rotmatrix0);
                            for (int i=0; i<3; i++) {
                                for (int j=0; j<3; j++) {
                                    rotmatrix[4*i+j]=weight0*rotmatrix[4*i+j]+weight1*rotmatrix0[4*i+j];
                                }
                            }
                        }
                    }
                }
            }
            public void onAccuracyChanged (Sensor sensor, int accuracy) {}
        };
    }

    //センサをチェックして起動する
    //センサモードを返す　0:センサ無し，1：ROTATION_VECTORセンサ利用　2：MAGNETIC_FIELDセンサとACCELEROMETERセンサを利用
    public int setSensorsOn() {
        firsttime = true;
        isReady=0;
        sensormode=0;
        Matrix.setIdentityM(rotmatrix, 0);
        Matrix.setIdentityM(rotmatrix0, 0);

        surfacerotation = getRotationValue();
        switch (surfacerotation) {
            case 0:
                //そのまま 端末を立ててportrait
                x_axis=SensorManager.AXIS_X;
                y_axis=SensorManager.AXIS_Y;
                break;
            case 90:
                // ワールド座標とデバイス座標のマッピングを変換する 端末を立ててlandscape（portrait時の上部を左に倒した）
                x_axis=SensorManager.AXIS_Y;
                y_axis=SensorManager.AXIS_MINUS_X;
                break;
            case 180:
                // ワールド座標とデバイス座標のマッピングを変換する 端末を立てて上下逆のportrait
                x_axis=SensorManager.AXIS_MINUS_X;
                y_axis=SensorManager.AXIS_MINUS_Y;
                break;
            case 270:
                // ワールド座標とデバイス座標のマッピングを変換する 端末を立てて上下逆のlandscape（portrait時の上部を右に倒した）
                x_axis=SensorManager.AXIS_MINUS_Y;
                y_axis=SensorManager.AXIS_X;
                break;
        }
        int sensoravailable=0;
        Sensor sensor;

        sensor = mSensorManager.getDefaultSensor( Sensor.TYPE_GYROSCOPE );
        if (sensor!=null) {
            mSensorManager.registerListener(mSensorEventListener, sensor, SensorManager.SENSOR_DELAY_UI);
            sensoravailable |= 2;

            sensor = mSensorManager.getDefaultSensor( Sensor.TYPE_ROTATION_VECTOR );
            if (sensor!=null) {
                mSensorManager.registerListener(mSensorEventListener, sensor, SensorManager.SENSOR_DELAY_UI);
                sensoravailable |= 1;
            }
            if ((sensoravailable&3) == 3 ) {
                sensormode=1;
                return sensormode;
            }
        }

        sensor = mSensorManager.getDefaultSensor( Sensor.TYPE_MAGNETIC_FIELD );
        if (sensor!=null) {
            mSensorManager.registerListener(mSensorEventListener, sensor, SensorManager.SENSOR_DELAY_UI);
            sensoravailable |= 4;
        }
        sensor = mSensorManager.getDefaultSensor( Sensor.TYPE_ACCELEROMETER );
        if (sensor!=null) {
            mSensorManager.registerListener(mSensorEventListener, sensor, SensorManager.SENSOR_DELAY_UI);
            sensoravailable |= 8;
        }
        if ((sensoravailable&0xc) == 0xc ) {
            sensormode=2;
            return sensormode;
        }
        return sensormode;
    }

    public void setSensorsOff() {
        //センサーリスナーを解除
        mSensorManager.unregisterListener( mSensorEventListener );
    }

    public float[] getRotMatrix() {
        float[] mat=rotmatrix.clone();
        return mat;
    }

    public int getSensorMode() {
        return sensormode;
    }

    private float getNorm(float[] vec) {
        double v2=(double)(vec[0]*vec[0]+vec[1]*vec[1]+vec[2]*vec[2]);
        return (float)Math.sqrt(v2);
    }

    float[] accpr = {0f,0f,0f};
    float[] magpr = {0f,0f,0f};
    float filtvalue=0.5f;
    double accnpr=0;
    double magnpr=0;
    private float getMovement(float[] acc, float[] mag) {
        float[] dacc = new float[4];
        float[] dmag = new float[4];
        double acc2,mag2,accn,magn;
        float ret;
        for (int i=0; i<3; i++) {
            dacc[i] = acc[i] - accpr[i];
            accpr[i]=acc[i];
            dmag[i] = mag[i] - magpr[i];
            magpr[i]=mag[i];
        }
        acc2=(double)(dacc[0]*dacc[0]+dacc[1]*dacc[1]+dacc[2]*dacc[2]);
        if (acc2<1e-4) {
            accn = accnpr;
        } else {
            accn=Math.sqrt(acc2);
            accnpr=accn;
        }
        mag2=(double)(dmag[0]*dmag[0]+dmag[1]*dmag[1]+dmag[2]*dmag[2])/5;
        if (mag2<1e-4) {
            magn=magnpr;
        } else {
            magn = Math.sqrt(mag2);
            magnpr=magn;
        }
        Log.i("RotationSEnsor","accn, magn = "+ accn + ",       " + magn);
        ret=(float)(accn+magn);
        ret=.7f*filtvalue+0.3f*ret;
        filtvalue=ret;
        return ret;
    }

    private float checkOrthogonalMatrix(float[] mat) {
        float[] tmat = new float[16];
        float[] mult = new float[16];
        Matrix.transposeM(tmat,0,mat,0);
        Matrix.multiplyMM(mult,0,mat,0,tmat,0);
        float[] im= new float[16];
        Matrix.setIdentityM(im,0);
        float ret=0f;
        for (int i=0; i<16; i++) {
            float elm=mult[i]-im[i];
            ret += elm*elm;
        }
        return ret;
    }
    private void changeSensor() {
        setSensorsOff();
        firsttime = true;
        isReady=0;
        sensormode=0;
        Matrix.setIdentityM(rotmatrix, 0);
        Matrix.setIdentityM(rotmatrix0, 0);

        int sensoravailable=0;
        Sensor sensor;

        sensor = mSensorManager.getDefaultSensor( Sensor.TYPE_MAGNETIC_FIELD );
        if (sensor!=null) {
            mSensorManager.registerListener(mSensorEventListener, sensor, SensorManager.SENSOR_DELAY_UI);
            sensoravailable |= 4;
        }
        sensor = mSensorManager.getDefaultSensor( Sensor.TYPE_ACCELEROMETER );
        if (sensor!=null) {
            mSensorManager.registerListener(mSensorEventListener, sensor, SensorManager.SENSOR_DELAY_UI);
            sensoravailable |= 8;
        }
        if ((sensoravailable&0xc) == 0xc ) {
            sensormode=2;
        }
    }

    //端末の画面の向きを取得する。
    //返す値 画面の回転角度
    public int getRotationValue() {
        int val = -1;
        //==== 画面向き取得 ====//
        int rotation = mDisplay.getRotation();
        switch(rotation) {
            case Surface.ROTATION_0:
                //端末を立ててportrait　背面カメラでportrait（縦位置）写真をとる状態
                val = 0;
                break;
            case Surface.ROTATION_90:
                //端末を立ててportrait状態から左に90度倒したlandscape　背面カメラでlandscape（横位置）写真をとる状態
                val = 90;
                break;
            case Surface.ROTATION_180:
                //端末を立ててportrait状態から180度倒した逆portrait
                val = 180;
                break;
            case Surface.ROTATION_270:
                //端末を立ててportrait状態から右に90度倒した逆landscape
                val = 270;
                break;
        }
        return val;
    }


}
