package jp.gr.java_conf.coskx.gles2sample09r2;

import android.opengl.GLES20;

import java.nio.FloatBuffer;
import java.nio.ShortBuffer;

/**
 * Created by tommy on 2017/06/30.
 */

public class WireSphere {
    //bufferの定義
    private FloatBuffer vertexBuffer;
    private ShortBuffer indexBuffer;

    private int nIndexs;

    WireSphere(){makeSphere(1f, 20, 10);}
    WireSphere(int nSlices, int nStacks){makeSphere(1f, nSlices, nStacks);}
    public void makeSphere(float Radius, int nSlices, int nStacks) {
        //頂点座標
        int sizeArray=((nStacks-1)*nSlices+2)*3;
        float[] vertexs= new float[sizeArray];
        vertexs[0]=0f; vertexs[1]=Radius; vertexs[2]=0f;
        int i,j,px;
        float theta,phi;
        for (i=0;i<nStacks-1;i++) {
            for (j=1;j<=nSlices;j++) {
                px=(i*nSlices+j)*3;
                theta=(float)(nStacks-i-1)/(float)nStacks*3.14159265f-3.14159265f*0.5f;
                phi=(float)j/(float)nSlices*2.f*3.14159265f;
                vertexs[px]=(float)(Radius* Math.cos(theta)* Math.sin(phi));
                vertexs[px+1]=(float)(Radius* Math.sin(theta));
                vertexs[px+2]=(float)(Radius* Math.cos(theta)* Math.cos(phi));
            }
        }
        px=((nStacks-1)*nSlices+1)*3;
        vertexs[px]=0f; vertexs[px+1]=-Radius; vertexs[px+2]=0f;

        //拡頂点の法線ベクトル
        //一般的には必要だが，球では頂点座標ベクトルと法線ベクトルが一致するため使わない
        //float[] normals= new float[sizeArray];
        //for (i=0;i<sizeArray;i++) normals[i]=vertexs[i]/Radius;

        //頂点座標番号列
        nIndexs=(nStacks*nSlices+nSlices*(nStacks-1))*2;
        short [] indexs= new short[nIndexs];
        int p=0;

        for (i=0;i<nSlices;i++) {
            for (j=0;j<nStacks;j++) {
                if (j==0) {
                    indexs[p++]=0;
                    indexs[p++]=(short)(i+1);
                    indexs[p++]=(short)(i+1);
                    indexs[p++]=(short)((i+1)%nSlices+1);
                } else if (j==nStacks-1) {
                    indexs[p++]=(short)((j-1)*nSlices+i+1);
                    indexs[p++]=(short)((nStacks-1)*nSlices+1);
                } else {
                    indexs[p++]=(short)((j-1)*nSlices+i+1);
                    indexs[p++]=(short)(j*nSlices+i+1);
                    indexs[p++]=(short)(j*nSlices+i+1);
                    indexs[p++]=(short)(j*nSlices+(i+1)%nSlices+1);
                }
            }
        }

        vertexBuffer=BufferUtil.makeFloatBuffer(vertexs);
        indexBuffer=BufferUtil.makeShortBuffer(indexs);
    }

    public void draw(float r,float g,float b,float a){
        //頂点点列
        GLES20.glVertexAttribPointer(GLES.positionHandle, 3,
                GLES20.GL_FLOAT, false, 0, vertexBuffer);

        //単色の設定 (r, g, b,a)
        GLES20.glUniform4f(GLES.objectColorHandle, r, g, b, a);

        //描画をシェーダに指示
        indexBuffer.position(0);
        GLES20.glDrawElements(GLES20.GL_LINES,
                nIndexs, GLES20.GL_UNSIGNED_SHORT, indexBuffer);
    }
}
