package jp.gr.java_conf.coskx.gles2sample10c;

import android.view.MotionEvent;

/**
 * Created by tommy on 2015/09/08.
 */
public class GestureDetector {
    private GestureListener mListener;
    private int mPointerID1=-1, mPointerID2=-1;
    private float mOrgX=0f, mOrgY=0f;//初期値
    private float mPrevX1, mPrevY1;//前回のX1，Y1
    private float mPrevX2, mPrevY2;//前回のX2，Y2
    private float mCurnX1, mCurnY1;//今回のX1，Y1
    private float mCurnX2, mCurnY2;//今回のX2，Y2
    private float mDeltaX, mDeltaY;//１本指の時の移動量
    private float mDelta2X, mDelta2Y, mRotation;//2本指の時の移動量
    private float mFactor;//2本指の時の拡大率
    private long mBeginTime=0,mEndTime2=1;

    private final long TimeConstant=400;  //シングルショートタップ時間　[msec]
    private final long TimeConstant2=2000;  //最短シングルロングタップ時間　[msec]
    private final long TimeConstant3=4000;  //最長シングルロングタップ時間　[msec]
    private final float DisplacementLimit=200f;  //ロングタップ時の移動許容定義(距離の２乗)

    public GestureDetector(GestureListener listener)
    {
        mListener = listener;
    }

    // タッチ処理　２本指まで対応する
    // mPointerID1が常に優先
    public boolean onTouchEvent(MotionEvent event) {
        int ActionMasked = event.getActionMasked();
        int ActionIndex = event.getActionIndex();
        int pointerId = event.getPointerId(ActionIndex);
        float currentX = event.getX(ActionIndex);
        float currentY = event.getY(ActionIndex);
        long EventTime = event.getEventTime();

        switch (ActionMasked) {
            case MotionEvent.ACTION_DOWN://最初のタッチ
                mPointerID1 = pointerId;
                mPointerID2 = -1;
                mOrgX = mCurnX1 = mPrevX1 = currentX;
                mOrgY = mCurnY1 = mPrevY1 = currentY;
                mBeginTime=EventTime;
                mListener.onFirstTouch(this);
                break;

            case MotionEvent.ACTION_POINTER_DOWN://2つ目以降のタッチ
                if (mPointerID1 != -1 && mPointerID2 == -1) {
                    mPointerID2 = pointerId;
                    mCurnX2 = mPrevX2 = currentX;
                    mCurnY2 = mPrevY2 = currentY;
                    int ptrIndex = event.findPointerIndex(mPointerID1);
                    mCurnX1 = mPrevX1 = event.getX(ptrIndex);
                    mCurnY1 = mPrevY1 = event.getY(ptrIndex);
                    mBeginTime=EventTime;
                    mListener.onSecondTouch(this);
                }
                break;

            case MotionEvent.ACTION_POINTER_UP://２つ目以降のタッチの終了
                if (mPointerID1 == pointerId) {
                    if (mPointerID2 != -1) {
                        mPointerID1 = mPointerID2;
                        mCurnX1 = mPrevX1 = mPrevX2;
                        mCurnY1 = mPrevY1 = mPrevY2;
                        mPointerID2 = -1;
                        mEndTime2 = mBeginTime = EventTime;
                        mListener.onSecondTouchEnd(this);
                    } else {
                        mPointerID1 = -1;
                    }
                } else if (mPointerID2 == pointerId) {
                    mPointerID2 = -1;
                    if (mPointerID1 != -1) {
                        mEndTime2 = mBeginTime = EventTime;
                        mListener.onSecondTouchEnd(this);
                    }
                }
                break;

            case MotionEvent.ACTION_UP://タッチすべて終了
            case MotionEvent.ACTION_CANCEL://キャンセル
                if (mPointerID1 == pointerId && mEndTime2!=mBeginTime) {
                    long deltat= EventTime - mBeginTime;
                    float dx = mOrgX - currentX;
                    float dy = mOrgY - currentY;
                    float d2=dx*dx+dy*dy;
                    if (deltat<TimeConstant && d2<DisplacementLimit) {
                        mListener.onShortTap(this);
                    } else if (TimeConstant2<deltat && deltat<TimeConstant3 && d2<DisplacementLimit) {
                        mListener.onLongTap(this);
                    }
                }
                mListener.onFirstTouchEnd(this);
                mPointerID1 = -1;
                mPointerID2 = -1;
                break;

            case MotionEvent.ACTION_MOVE://指が移動中
                if(mPointerID1 != -1 && mPointerID2 != -1) {//２本指が移動中
                    mCurnX1 = event.getX(event.findPointerIndex(mPointerID1));
                    mCurnY1 = event.getY(event.findPointerIndex(mPointerID1));
                    mCurnX2 = event.getX(event.findPointerIndex(mPointerID2));
                    mCurnY2 = event.getY(event.findPointerIndex(mPointerID2));

                    float angle1 = calcAngle(mPrevX1, mPrevY1, mPrevX2, mPrevY2);
                    float angle2 = calcAngle(mCurnX1, mCurnY1, mCurnX2, mCurnY2);
                    float length1 = calcLength(mPrevX1, mPrevY1, mPrevX2, mPrevY2);
                    float length2 = calcLength(mCurnX1, mCurnY1, mCurnX2, mCurnY2);

                    mRotation = angle2 - angle1;
                    if (mRotation < -180.0f) {
                        mRotation += 360.0f;
                    } else if (mRotation > 180.0f) {
                        mRotation -= 360.0f;
                    }

                    mDelta2X = calcCenter(mCurnX1, mCurnX2) - calcCenter(mPrevX1, mPrevX2);
                    mDelta2Y = calcCenter(mCurnY1, mCurnY2) - calcCenter(mPrevY1, mPrevY2);

                    if (0<length1) {
                        mFactor = length2 / length1;
                    } else {
                        mFactor = 1f;
                    }

                    mPrevX1 = mCurnX1;
                    mPrevY1 = mCurnY1;
                    mPrevX2 = mCurnX2;
                    mPrevY2 = mCurnY2;

                    mListener.onTwoFingerMove(this);
                } else if (mPointerID1 == pointerId) {//１本指が移動中
                    mCurnX1 = currentX;
                    mCurnY1 = currentY;
                    mDeltaX = mCurnX1 - mPrevX1;
                    mDeltaY = mCurnY1 - mPrevY1;
                    mPrevX1 = mCurnX1;
                    mPrevY1 = mCurnY1;

                    mListener.onOneFingerMove(this);
                }
                break;
        }
        return true;
    }

    // 中点を計算する。
    private float calcCenter(float p1, float p2)
    {
        return (p1 + p2) / 2;
    }

    // 線分から角度[deg]を計算する。
    private float calcAngle(float x1, float y1, float x2, float y2)
    {
        return (float)Math.toDegrees(Math.atan2((y2 - y1), (x2 - x1)));
    }

    // 線分長を計算する。
    private float calcLength(float x1, float y1, float x2, float y2)
    {
        float dx = x2-x1;
        float dy = y2-y1;
        return (float)Math.sqrt(dx * dx + dy * dy);
    }

    public float getX1()
    {
        return mCurnX1;
    }

    public float getY1()
    {
        return mCurnY1;
    }

    public float getX2()
    {
        return mCurnX2;
    }

    public float getY2()
    {
        return mCurnY2;
    }

    //１本指スワイプ時のｘ方向移動量
    public float getDeltaX()
    {
        return mDeltaX;
    }

    //１本指スワイプ時のｙ方向移動量
    public float getDeltaY()
    {
        return mDeltaY;
    }

    //２本指スワイプ時のｘ方向移動量
    public float getDelta2X()
    {
        return mDelta2X;
    }

    //２本指スワイプ時のｙ方向移動量
    public float getDelta2Y()
    {
        return mDelta2Y;
    }

    //２本指スワイプ時の回転量[deg]
    public float getRotation()
    {
        return mRotation;
    }

    //２本指スワイプ時の拡大率
    public float getFactor()
    {
        return mFactor;
    }

}
