package jp.gr.java_conf.coskx.gles2sample11g;

import android.opengl.GLES20;

import java.nio.ByteBuffer;
import java.nio.FloatBuffer;

/**
 * Created by tommy on 2015/12/08.
 * zx平面にグラデーション付きの半径1の円を描く
 * 円の始点はｚ軸上で，始点の色はcolor0，終点の色はcolor1
 * 色はr,g,b,aで定義される要素数4のfloat配列である
 */
public class CircleGradation {
    //buffer
    private FloatBuffer vertexBuffer;
    private FloatBuffer colorBuffer;
    private ByteBuffer indexBuffer;

    private int NumberParts;

    CircleGradation(int NumParts, float[] color0, float[] color1){
        makeCircle(1f, NumParts, color0, color1);
    } //Max 255
    public void makeCircle(float Radius, int NumParts, float[] color0, float[] color1) {
        float[] vertexs=new float[(NumParts+1)*3];
        float[] colors=new float[(NumParts+1)*4];
        byte[] indexs=new byte[NumParts+1];
        short i,j;
        NumberParts=NumParts;
        float theta;
        for (i=0; i<=NumParts; i++) {
            theta=(float)(i * 2 * Math.PI / (float)NumParts);
            vertexs[3*i]=(float)(Radius*Math.sin(theta));
            vertexs[3*i+1]=0f;
            vertexs[3*i+2]=(float)(Radius*Math.cos(theta));
            indexs[i]=(byte)i;
        }
        for (i=0; i<4; i++) {
            float dcolor = (color1[i]-color0[i])/(float)NumParts;
            for (j=0; j<=NumParts; j++) {
                colors[4*j+i] = color0[i]+dcolor*(float)j;
            }
        }
        vertexBuffer = BufferUtil.makeFloatBuffer(vertexs);
        colorBuffer = BufferUtil.makeFloatBuffer(colors);
        indexBuffer = BufferUtil.makeByteBuffer(indexs);
    }
    public void draw(float linewidth){
        //頂点点列
        GLES20.glVertexAttribPointer(GLES.positionHandle, 3,
                GLES20.GL_FLOAT, false, 0, vertexBuffer);

        //頂点色列
        GLES20.glVertexAttribPointer(GLES.colorHandle, 4,
                GLES20.GL_FLOAT, false, 0, colorBuffer);

        //線の太さ
        GLES20.glLineWidth(linewidth);

        //描画をシェーダに指示
        indexBuffer.position(0);
        GLES20.glDrawElements(GLES20.GL_LINE_STRIP,
                NumberParts+1, GLES20.GL_UNSIGNED_BYTE, indexBuffer);
    }
}
