package jp.gr.java_conf.coskx.gles2sample11h;

import android.opengl.GLES20;

import java.nio.FloatBuffer;
import java.nio.ShortBuffer;

/**
 * Created by tommy on 2016/09/07.
 */
public class PlaneRing {
    //bufferの定義
    private FloatBuffer vertexBuffer;
    private ShortBuffer indexBuffer;
    private FloatBuffer normalBuffer;

    private int nIndexs;

    //zx平面上に置かれた平面リングのクラス
    PlaneRing(){}
    //外側の半径，内側の半径，台形近似する台形の数　の3つを引数にとったコンストラクタ
    PlaneRing(float RadiusOuter, float RadiusInner, int nSlices){makePlaneRing(RadiusOuter,RadiusInner,nSlices,1);}
    //外側の半径，内側の半径，台形近似する台形の数，台形を高さ方向分割する数　の4つを引数にとったコンストラクタ
    PlaneRing(float RadiusOuter, float RadiusInner, int nSlices, int nStacks){makePlaneRing(RadiusOuter,RadiusInner,nSlices,nStacks);}
    public void makePlaneRing(float RadiusOuter, float RadiusInner, int nSlices, int nStacks) {
        //頂点座標
        int numPoints = (nSlices+1)*(nStacks+1);
        nIndexs = nSlices*(nStacks+1)*2;
        int sizeArray=numPoints*3;
        float[] vertexs= new float[sizeArray];
        float[] normals= new float[sizeArray];
        short [] indexs= new short[nIndexs];
        int i,j;
        double theta0=2.0*3.1415956535/nSlices;
        double theta;
        double dr = (RadiusOuter-RadiusInner)/nStacks;
        double r;
        int p=0;
        for (i=0; i<=nSlices; i++) {
            theta = theta0 * i;
            for (j = 0; j <= nStacks; j++) {
                r = (RadiusOuter - j * dr);
                vertexs[p++] = (float) (r * Math.sin(theta));
                vertexs[p++] = 0f;
                vertexs[p++] = (float) (r * Math.cos(theta));
            }
        }
        p=0;
        for (i=0; i<=nSlices; i++) {
            for (j = 0; j <= nStacks; j++) {
                normals[p++] = 0f;
                normals[p++] = 1f;
                normals[p++] = 0f;
            }
        }
        p=0;
        int nStacks1=nStacks+1;
        for (i=0; i<nSlices; i++) {
            for (j = 0; j <= nStacks; j++) {
                indexs[p++] = (short)(i*nStacks1+j);
                indexs[p++] = (short)((i+1)*nStacks1+j);
            }
        }
        vertexBuffer=BufferUtil.makeFloatBuffer(vertexs);
        indexBuffer=BufferUtil.makeShortBuffer(indexs);
        normalBuffer=BufferUtil.makeFloatBuffer(normals);
    }
    /**
     * 4 9 14 19
     * 3 8 13 18
     * 2 7 12 17
     * 1 6 11 16
     * 0 5 10 15    nSlices=3 nStacks=4
     *
     * 8 9 18 19 28 29
     * 6 7 16 17 26 27
     * 4 5 14 15 24 25
     * 2 3 12 13 22 23
     * 0 1 10 11 20 21
    **/
    public void draw(float r,float g,float b,float a, float shininess){
        //頂点点列
        GLES20.glVertexAttribPointer(GLES.positionHandle, 3,
                GLES20.GL_FLOAT, false, 0, vertexBuffer);

        if (GLES.checkLiting()) {
            //頂点での法線ベクトル
            GLES20.glVertexAttribPointer(GLES.normalHandle, 3,
                GLES20.GL_FLOAT, false, 0, normalBuffer);

            //周辺光反射
            GLES20.glUniform4f(GLES.materialAmbientHandle, r, g, b, a);

            //拡散反射
            GLES20.glUniform4f(GLES.materialDiffuseHandle, r, g, b, a);

            //鏡面反射
            GLES20.glUniform4f(GLES.materialSpecularHandle, r, g, b, a);
            GLES20.glUniform1f(GLES.materialShininessHandle, shininess);

        } else {
            //shadingを使わない時に使う単色の設定 (r, g, b,a)
            GLES20.glUniform4f(GLES.objectColorHandle, r, g, b, a);
        }

        //描画をシェーダに指示
        indexBuffer.position(0);
        GLES20.glDrawElements(GLES20.GL_TRIANGLE_STRIP,
                nIndexs, GLES20.GL_UNSIGNED_SHORT, indexBuffer);

    }
}
