package jp.gr.java_conf.coskx.gles2sample11h;

import android.opengl.GLES20;

import java.nio.ByteBuffer;
import java.nio.FloatBuffer;
import java.nio.ShortBuffer;

/**
 * Created by tommy on 2016/09/07.
 */
public class TexPlaneRing {
    //bufferの定義
    private FloatBuffer vertexBuffer;
    private ShortBuffer indexBuffer;
    private FloatBuffer normalBuffer;
    private FloatBuffer texcoordBuffer;
    private int nIndexs;

    //zx平面上に置かれた平面リングのクラス
    TexPlaneRing(){}
    //外側の半径，内側の半径，台形近似する台形の数　の3つを引数にとったコンストラクタ
    TexPlaneRing(float RadiusOuter, float RadiusInner, int nSlices){makePlaneRing(RadiusOuter,RadiusInner,nSlices,1);}
    //外側の半径，内側の半径，台形近似する台形の数，台形を高さ方向分割する数　の4つを引数にとったコンストラクタ
    TexPlaneRing(float RadiusOuter, float RadiusInner, int nSlices, int nStacks){makePlaneRing(RadiusOuter,RadiusInner,nSlices,nStacks);}
    public void makePlaneRing(float RadiusOuter, float RadiusInner, int nSlices, int nStacks) {
        //頂点座標
        int numPoints = (nSlices+1)*(nStacks+1);
        nIndexs = nSlices*(nStacks+1)*2;
        int sizeArray=numPoints*3;
        float[] vertexs= new float[sizeArray];
        float[] normals= new float[sizeArray];
        short [] indexs= new short[nIndexs];
        int i,j;
        double theta0=2.0*3.1415956535/nSlices;
        double theta;
        double dr = (RadiusOuter-RadiusInner)/nStacks;
        double r;
        int p=0;
        for (i=0; i<=nSlices; i++) {
            theta = theta0 * i;
            for (j = 0; j <= nStacks; j++) {
                r = (RadiusOuter - j * dr);
                vertexs[p++] = (float) (r * Math.sin(theta));
                vertexs[p++] = 0f;
                vertexs[p++] = (float) (r * Math.cos(theta));
            }
        }
        p=0;
        for (i=0; i<=nSlices; i++) {
            for (j = 0; j <= nStacks; j++) {
                normals[p++] = 0f;
                normals[p++] = 1f;
                normals[p++] = 0f;
            }
        }
        p=0;
        int nStacks1=nStacks+1;
        for (i=0; i<nSlices; i++) {
            for (j = 0; j <= nStacks; j++) {
                indexs[p++] = (short)(i*nStacks1+j);
                indexs[p++] = (short)((i+1)*nStacks1+j);
            }
        }

        sizeArray=numPoints*2;
        float textcoords[] = new float[sizeArray];
        float dx = 1f/(float)nSlices;
        float dy = 1f/(float)nStacks;
        p = 0;
        for (i=0; i <= nSlices; i++) {
            for (j = 0; j <= nStacks; j++) {
                textcoords[p++] = i * dx;        // x
                textcoords[p++] = 1f - j * dy;   // y
            }
        }
        texcoordBuffer = BufferUtil.makeFloatBuffer(textcoords);

        vertexBuffer=BufferUtil.makeFloatBuffer(vertexs);
        indexBuffer=BufferUtil.makeShortBuffer(indexs);
        normalBuffer=BufferUtil.makeFloatBuffer(normals);
        texcoordBuffer = BufferUtil.makeFloatBuffer(textcoords);
    }

    public void draw(float r,float g,float b,float a, float shininess){
        //頂点点列のテクスチャ座標
        GLES20.glVertexAttribPointer(GLES.texcoordHandle, 2,
                GLES20.GL_FLOAT, false, 0, texcoordBuffer);
        //頂点点列
        GLES20.glVertexAttribPointer(GLES.positionHandle, 3,
                GLES20.GL_FLOAT, false, 0, vertexBuffer);

        if (GLES.checkLiting()) {
            //頂点での法線ベクトル
            GLES20.glVertexAttribPointer(GLES.normalHandle, 3,
                    GLES20.GL_FLOAT, false, 0, normalBuffer);

            //周辺光反射
            GLES20.glUniform4f(GLES.materialAmbientHandle, r, g, b, a);

            //拡散反射
            GLES20.glUniform4f(GLES.materialDiffuseHandle, r, g, b, a);

            //鏡面反射
            GLES20.glUniform4f(GLES.materialSpecularHandle, r, g, b, a);
            GLES20.glUniform1f(GLES.materialShininessHandle, shininess);

        } else {
            //shadingを使わない時に使う単色の設定 (r, g, b,a)
            GLES20.glUniform4f(GLES.objectColorHandle, r, g, b, a);
        }

        //描画をシェーダに指示
        indexBuffer.position(0);
        GLES20.glDrawElements(GLES20.GL_TRIANGLE_STRIP,
                nIndexs, GLES20.GL_UNSIGNED_SHORT, indexBuffer);

    }

}
