package jp.gr.java_conf.coskx.gles2sample14;

import android.content.Context;
import android.opengl.GLES20;
import android.opengl.GLSurfaceView;
import android.opengl.Matrix;
import android.os.Handler;
import android.text.format.DateFormat;
import android.widget.TextView;

import java.util.Calendar;
import java.util.TimeZone;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;


/**
 * Created by tommy on 2015/06/18.
 */
public class GLRenderer implements GLSurfaceView.Renderer {
    //システム
    private final Context mContext;
    private TextView mTextViewTime;
    private Handler mHandler = new Handler();
    private boolean validProgram=false; //シェーダプログラムが有効
    private float aspect;//アスペクト比

    private float viewlength0 = 5.0f; //視点距離
    private float viewlength = 5.0f; //視点距離
    private float viewingangle0 = 45f; //視野角（画面長手方向に対応する視野角[deg]）
    private float viewingangle = 45f; //視野角（画面長手方向に対応する視野角[deg]）
    private float RotAngle=0f;       //2本指回転量
    private float AngleHor=0f;       //横移動角（2本指）[deg]
    private float AngleVer=0f;       //縦移動角（2本指）[deg]

    private float   angle=0f; //回転角度
    private float rotatatinvelocity=1.f;

    //視点変更テスト変数
    private float alph=0f,beta=0f;

    //光源の座標　x,y,z
    private  float[] LightPos={0f,1.5f,3f,1f};//x,y,z,1
    //光源の環境光
    private float[] LightAmb={.15f,.15f,.15f,1f}; //r,g,b,a
    //光源の乱反射光
    private float[] LightDif={.5f,.5f,.5f,1f}; //r,g,b,a
    //光源の鏡面反射反射光
    private float[] LightSpc={.9f,.9f,.9f,1f}; //r,g,b,a

    //変換マトリックス
    private  float[] pMatrix=new float[16]; //プロジェクション変換マトリックス
    private  float[] mMatrix=new float[16]; //モデル変換マトリックス
    private  float[] cMatrix=new float[16]; //カメラビュー変換マトリックス

    //モデル座標系の原点
    private  float[] origin= {0f,0f,0f,1f};

    private Axes MyAxes= new Axes();  //原点周囲の軸表示とためのオブジェクトを作成
    private SphereEX mHemisphere = new SphereEX(360,0,90,64,16);
    private ConeEX mCylinder = new ConeEX(1f,1f,360f,64);
    private ConeEX mDisk = new ConeEX(0f,0f,360f,64);
    private TorusEX m1TorusEX = new TorusEX(.5f, 360f,0f,360f,72,36);
    private TorusEX m2TorusEX = new TorusEX(.5f, 360f,0f,180f,72,36); //上半分トーラスを定義
    private Droid mDroid = new Droid();

    private float[] c0 = {0f,0f,1f,1f};
    private float[] c1 = {0f,1f,1f,1f};
    private CircleEX MyCircle = new CircleEX(270,64,c0,c1); //半径１の270度のグラド円弧

    GLRenderer(final Context context) {
        mContext = context;
    }

    //サーフェイス生成時に呼ばれる
    @Override
    public void onSurfaceCreated(GL10 gl10,EGLConfig eglConfig) {
        //プログラムの生成
        validProgram = GLES.makeProgram();

        //デプスバッファの有効化
        GLES20.glEnable(GLES20.GL_DEPTH_TEST);

        // カリングの有効化
        GLES20.glEnable(GLES20.GL_CULL_FACE); //裏面を表示しないチェックを行う

        // 裏面を描画しない
        GLES20.glFrontFace(GLES20.GL_CCW); //表面のvertexのindex番号はCCWで登録
        GLES20.glCullFace(GLES20.GL_BACK); //裏面は表示しない

        //背景色の設定
        GLES20.glClearColor(0f, 0f, 0.2f, 1.0f);

        //光源の設定
        GLES.putLightAttribute(LightAmb, LightDif, LightSpc);

        // 背景とのブレンド方法を設定します。
        GLES20.glEnable(GLES20.GL_BLEND);
        GLES20.glBlendFunc(GLES20.GL_SRC_ALPHA, GLES20.GL_ONE_MINUS_SRC_ALPHA);    // 単純なアルファブレンド
    }

    //画面サイズ変更時に呼ばれる
    @Override
    public void onSurfaceChanged(GL10 gl10,int w,int h) {
        //ビューポート変換
        GLES20.glViewport(0,0,w,h);
        aspect=(float)w/(float)h;
    }

    //毎フレーム描画時に呼ばれる
    @Override
    public void onDrawFrame(GL10 glUnused) {
        if (!validProgram) return;

        //TextViiewはUIスレッドなので別スレッドを起動要求
        //画面下部へのJSTの表示
        mHandler.post(new Runnable() {
            @Override
            public void run() {
                String dateString = (String) DateFormat.format(
                        "yyyy-MM-dd kk:mm:ss",
                        Calendar.getInstance(TimeZone.getTimeZone("Asia/Tokyo")));
                mTextViewTime.setText(dateString);
                return;
            }
        });

        //画面のクリア
        GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT |
                GLES20.GL_DEPTH_BUFFER_BIT);

        //プロジェクション変換（射影変換）--------------------------------------
        //透視変換（遠近感を作る）
        //カメラは原点に有り，z軸の負の方向を向いていて，上方向はy軸＋方向である。
        GLES.gluPerspective(pMatrix,
                viewingangle,  //Y方向の画角
                aspect, //アスペクト比
                //1.0f,   //ニアクリップ　　　z=-1から
                0.00001f,   //ニアクリップ　　　z=-0.00001から
                100.0f);//ファークリップ　　Z=-100までの範囲を表示することになる
        GLES.setPMatrix(pMatrix);

        //カメラビュー変換（視野変換）-----------------------------------
        //カメラ視点が原点になるような変換
        float[] c1Matrix=new float[16]; //カメラ視点変換マトリックス作成用
        float[] c2Matrix=new float[16]; //カメラ視点変換マトリックス作成用
        Matrix.setLookAtM(c1Matrix, 0,
                (float) (viewlength * Math.sin(beta) * Math.cos(alph)),  //カメラの視点 x
                (float) (viewlength * Math.sin(alph)),                    //カメラの視点 y
                (float) (viewlength * Math.cos(beta) * Math.cos(alph)),  //カメラの視点 z
                0.0f, 0.0f, 0.0f, //カメラの視線方向の代表点
                0.0f, 1.0f, 0.0f);//カメラの上方向
        Matrix.setIdentityM(c2Matrix, 0);
        Matrix.rotateM(c2Matrix, 0, -AngleHor, 0, 1, 0);
        Matrix.rotateM(c2Matrix, 0, -AngleVer, 1, 0, 0);
        Matrix.rotateM(c2Matrix, 0, -RotAngle, 0, 0, 1);
        Matrix.multiplyMM(cMatrix, 0, c2Matrix, 0, c1Matrix, 0); //cMatrix = c2Matrix * c1Matrix
        //カメラ視点ビュー変換はこれで終わり。
        GLES.setCMatrix(cMatrix);

        //cMatrixをセットしてから光源位置をセット
        GLES.setLightPosition(LightPos);

        GLES.selectProgram(GLES.SP_SimpleObject);

        //座標軸の描画
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        //座標軸の描画本体
        //引数 r, g, b, a, shininess(1以上の値　大きな値ほど鋭くなる), linewidth
        //shininessは使用していない
        MyAxes.draw(1f, 1f, 1f, 1f, 2f);//座標軸の描画本体

        GLES.selectProgram(GLES.SP_ObjectWithLight);

        //ドロイド君の描画１
        mDroid.drawDroid();
        //ドロイド君の描画２
        Matrix.setIdentityM(mMatrix, 0);
        Matrix.rotateM(mMatrix, 0, 1.5f * angle, 0, 1, 0);
        Matrix.translateM(mMatrix, 0, -1.2f, 0f, 0f);
        Matrix.rotateM(mMatrix, 0, 2f * angle, 0, 1, 0);
        Matrix.scaleM(mMatrix, 0, 0.5f, 0.5f, 0.5f);
        mDroid.drawDroid(mMatrix);
        //ドロイド君の描画３
        Matrix.setIdentityM(mMatrix, 0);
        Matrix.rotateM(mMatrix, 0, 1.5f * angle+45f, 0, 1, 0);
        Matrix.translateM(mMatrix, 0, -1.2f, 0f, 0f);
        Matrix.rotateM(mMatrix, 0, 2f * angle, 0, 0, 1);
        Matrix.scaleM(mMatrix, 0, 0.5f, 0.5f, 0.5f);
        mDroid.drawDroid(mMatrix);
        //ドロイド君の描画４
        Matrix.setIdentityM(mMatrix, 0);
        Matrix.rotateM(mMatrix, 0, 1.5f * angle-45f, 0, 1, 0);
        Matrix.translateM(mMatrix, 0, -1.2f, 0f, 0f);
        Matrix.rotateM(mMatrix, 0, 2f * angle, 1, 0, 0);
        Matrix.scaleM(mMatrix, 0, 0.5f, 0.5f, 0.5f);
        mDroid.drawDroid(mMatrix);

        //mHemisphereの描画
        Matrix.setIdentityM(mMatrix, 0);
        Matrix.rotateM(mMatrix, 0, 1.5f * angle, 0, 1, 0);
        Matrix.translateM(mMatrix, 0, 0f, 0f, 1.2f);
        Matrix.rotateM(mMatrix, 0, 2f * angle, 0, 1, 0);
        Matrix.scaleM(mMatrix, 0, 0.2f, 0.2f, 0.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        //mHemisphereの描画本体
        // r, g, b, a, shininess(1以上の値　大きな値ほど鋭くなる)
        mHemisphere.draw(1f, 1f, 0f, 1f, 5.f);
        //裏側は汚いです

        //mCylinderの描画
        Matrix.setIdentityM(mMatrix, 0);
        Matrix.rotateM(mMatrix, 0, 1.5f * angle, 0, 1, 0);
        Matrix.translateM(mMatrix, 0, 0f, 0f, -1.2f);
        Matrix.rotateM(mMatrix, 0, 2f * angle, 0, 1, 0);
        Matrix.scaleM(mMatrix, 0, 0.2f, 0.2f, 0.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        //mHemisphereの描画本体
        // r, g, b, a, shininess(1以上の値　大きな値ほど鋭くなる)
        mCylinder.draw(1f, 1f, 0f, 1f, 5.f);
        //裏側は汚いです

        //mDiskの描画
        Matrix.setIdentityM(mMatrix, 0);
        Matrix.rotateM(mMatrix, 0, 1.5f * angle + 45f, 0, 1, 0);
        Matrix.translateM(mMatrix, 0, 1.2f, 0f, 0f);
        Matrix.rotateM(mMatrix, 0, 2f * angle, 0, 1, 0);
        Matrix.scaleM(mMatrix, 0, 0.2f, 0.2f, 0.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        //mDiskの描画本体
        // r, g, b, a, shininess(1以上の値　大きな値ほど鋭くなる)
        mDisk.draw(1f, 1f, 0f, 1f, 5.f);
        //裏は見えません

        //m1TorusEXの描画
        Matrix.setIdentityM(mMatrix, 0);
        Matrix.rotateM(mMatrix, 0, 1.5f * angle, 0, 1, 0);
        Matrix.translateM(mMatrix, 0, 1.2f, 0f, 0f);
        Matrix.rotateM(mMatrix, 0, 2f * angle, 0, 1, 0);
        Matrix.scaleM(mMatrix, 0, 0.2f, 0.2f, 0.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        //m1TorusEXの描画本体
        // r, g, b, a, shininess(1以上の値　大きな値ほど鋭くなる)
        m1TorusEX.draw(.5f, .5f, 1f, 1f, 5.f);
        //裏は見えません

        //m2TorusEXの描画
        Matrix.setIdentityM(mMatrix, 0);
        Matrix.rotateM(mMatrix, 0, 1.5f * angle - 45f, 0, 1, 0);
        Matrix.translateM(mMatrix, 0, 1.2f, 0f, 0f);
        Matrix.rotateM(mMatrix, 0, 2f * angle, 0, 1, 0);
        Matrix.scaleM(mMatrix, 0, 0.2f, 0.2f, 0.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        //m2TorusEXの描画本体
        // r, g, b, a, shininess(1以上の値　大きな値ほど鋭くなる)
        m2TorusEX.draw(.5f, .5f, 1f, 1f, 5.f);
        //裏は見えません

        GLES.selectProgram(GLES.SP_SimpleObjectVTXColor);
        //円の描画
        Matrix.setIdentityM(mMatrix, 0);
        Matrix.rotateM(mMatrix, 0, 1.5f * angle, 0, 1, 0);
        Matrix.scaleM(mMatrix, 0, 1.2f, 1.2f, 1.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        //円の描画本体
        //  linewidth
        MyCircle.drawGradation(1f);

        angle+=0.5f*rotatatinvelocity;

    }

    public void setRotationVelocity(float rotv) {
        this.rotatatinvelocity=rotv;
    }
    public void setTextview(TextView tv) {
        mTextViewTime=tv;
    }

    //一本指でのスクロール
    private float Scroll[] = {0f, 0f}; //１本指のドラッグ[rad]
    public void setScrollValue(float DeltaX, float DeltaY) {
        Scroll[0] -= DeltaX * 0.01;
        if (3.14f < Scroll[0]) Scroll[0] = 3.14f;
        if (Scroll[0] < -3.14) Scroll[0] = -3.14f;
        Scroll[1] += DeltaY * 0.01;
        if (1.57f < Scroll[1]) Scroll[1] = 1.57f;
        if (Scroll[1] < -1.57) Scroll[1] = -1.57f;
        alph = Scroll[1];
        beta = Scroll[0];
    }


    //２本指でのスクロール
    private float Scroll2fg[] = {0f, 0f}; //２本指のドラッグ[deg]
    public void setScroll2Value(float Delta2X, float Delta2Y) {
        Scroll2fg[0] += Delta2X * 0.05; //[deg]
        Scroll2fg[1] += Delta2Y * 0.05; //[deg]
        AngleHor = Scroll2fg[0];
        AngleVer = Scroll2fg[1];
    }

    public void setScroll3Value(float Delta2X, float Delta2Y) {

    }

    //２本指でのピンチイン・ピンチアウト
    private float My_ScaleFactor = 1.0f;
    public void setPinch2(float factor) {
        My_ScaleFactor *= factor;
        viewlength = viewlength0 / My_ScaleFactor;
    }

    //３本指でのピンチイン・ピンチアウト
    private float My_ScaleFactor3 = 1.0f;
    public void setPinch3(float factor) {
        My_ScaleFactor3 *= factor;
        viewingangle = viewingangle0 / My_ScaleFactor3;
    }

    //２本指でのローテート
    private float My_RotAngle =0f;
    public void setRot2Angle(float Rotation) {
        My_RotAngle += Rotation;
        RotAngle = My_RotAngle;
    }

    public void setRot3Angle(float Rotation) {

    }

}
