package jp.gr.java_conf.coskx.gles2sample14p;

import android.opengl.GLES20;

import java.nio.FloatBuffer;
import java.nio.ShortBuffer;

/**
 * Created by tommy on 2016/10/11.
 * 半径１の円を描く　ただし拡張されていて，円弧も描ける
 * 円弧を描くことが一般的な使い方で，特別な場合として円も描ける
 * 単一色でも描けるし，両端の２色を指定しグラデーションを付けることもできる
 * 円弧，円はzx平面内に描く
 * 円弧の起点は(0,0,1)でｚ軸回りにangle[deg]だけ描く
 * 直線近似で描くが，短い線分の数はnParts個，点の数はn+1個
 */

public class CircleEX {
    //一般的な半径１の円を描く(色指定は描く時)
    CircleEX() {
        makeCircle(360,64);
    }
    //一般的な半径１，中心角angle[deg]，線分数nPartsの弧を描く(色指定は描く時)
    CircleEX(float angle, int nParts) {
        makeCircle(angle,nParts);
    }
    //２色指定で一般的な半径１，中心角angle[deg]，線分数nParts，グラデーションの弧を描く
    //color0:起点の色，color1:終点の色
    //color0,color1 はRGBAのfloat配列
    CircleEX(float angle, int nParts, float[] color0, float[] color1) {
        makeCircle(angle,nParts,color0,color1);
    }

    private int nPoints;
    private float[] PointColor;
    private float[] vertexs;
    private short[] indexes;

    //buffer
    private FloatBuffer vertexBuffer;
    private FloatBuffer colorBuffer;
    private ShortBuffer indexBuffer;

    private void makeCircle(float angle, int nParts){
        nPoints=nParts+1;
        vertexs=new float[(nPoints)*3];
        indexes=new short[nPoints];
        double theta=angle/nParts*3.141592653589793/180;
        int p=0;
        for (int i=0; i<nPoints; i++) {
            vertexs[p++] = (float) Math.sin(theta*i);
            vertexs[p++] = 0f;
            vertexs[p++] = (float) Math.cos(theta*i);
            indexes[i] = (short)i;
        }
        vertexBuffer = BufferUtil.makeFloatBuffer(vertexs);
        indexBuffer = BufferUtil.makeShortBuffer(indexes);
    }
    private void makeCircle(float angle, int nParts, float[] color0, float[] color1){
        //nPoints=nParts+1;
        makeCircle(angle,nParts);
        PointColor = new float[4*nPoints];
        for (int i=0; i<4; i++) {
            float dcolor = (color1[i]-color0[i])/(float)nParts;
            for (int j=0; j<nPoints; j++) {
                PointColor[4*j+i] = color0[i]+dcolor*(float)j;
            }
        }
        colorBuffer = BufferUtil.makeFloatBuffer(PointColor);
    }

    public void drawGradation() {
        drawGradation(1f);
    }

    public void drawGradation(float linewidth) {
        if (PointColor==null) return;
        //頂点点列
        GLES20.glVertexAttribPointer(GLES.positionHandle, 3,
                GLES20.GL_FLOAT, false, 0, vertexBuffer);

        //頂点色列
        GLES20.glVertexAttribPointer(GLES.colorHandle, 4,
                GLES20.GL_FLOAT, false, 0, colorBuffer);

        //線の太さ
        GLES20.glLineWidth(linewidth);

        //描画をシェーダに指示
        indexBuffer.position(0);
        GLES20.glDrawElements(GLES20.GL_LINE_STRIP,
                nPoints, GLES20.GL_UNSIGNED_SHORT, indexBuffer);
    }

    public void draw(float[] color) {
        draw(color[0],color[1],color[2],color[3]);
    }

    public void draw(float R, float G, float B, float A) {
        draw(R,G,B,A,1f);
    }

    public void draw(float[] color, float linewidth) {
        draw(color[0],color[1],color[2],color[3], linewidth);
    }

    public void draw(float R, float G, float B, float A,float linewidth) {
        if (PointColor!=null) return;
        //頂点点列
        GLES20.glVertexAttribPointer(GLES.positionHandle, 3,
                GLES20.GL_FLOAT, false, 0, vertexBuffer);

        //shadingを使わない時に使う単色の設定 (r, g, b,a)
        GLES20.glUniform4f(GLES.objectColorHandle, R,G,B,A);

        //線の太さ
        GLES20.glLineWidth(linewidth);

        //描画をシェーダに指示
        indexBuffer.position(0);
        GLES20.glDrawElements(GLES20.GL_LINE_STRIP,
                nPoints, GLES20.GL_UNSIGNED_SHORT, indexBuffer);
    }

    public void draw(float linewidth) {
        if (PointColor!=null) return;
        //頂点点列
        GLES20.glVertexAttribPointer(GLES.positionHandle, 3,
                GLES20.GL_FLOAT, false, 0, vertexBuffer);

        //shadingを使わない時に使う単色の設定 (r, g, b,a)
        GLES20.glUniform4f(GLES.objectColorHandle, 1f,1f,1f,1f);

        //線の太さ
        GLES20.glLineWidth(linewidth);

        //描画をシェーダに指示
        indexBuffer.position(0);
        GLES20.glDrawElements(GLES20.GL_LINE_STRIP,
                nPoints, GLES20.GL_UNSIGNED_SHORT, indexBuffer);
    }

}
