package jp.gr.java_conf.coskx.gles2sample14p;

import android.content.Context;
import android.opengl.GLES20;
import android.opengl.GLSurfaceView;
import android.opengl.Matrix;
import android.os.Handler;
import android.widget.TextView;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;


/**
 * Created by tommy on 2015/06/18.
 */
public class GLRenderer implements GLSurfaceView.Renderer {
    //システム
    private final Context mContext;
    private TextView mTextViewTime;
    private Handler mHandler = new Handler();
    private boolean validProgram=false; //シェーダプログラムが有効
    private float aspect;//アスペクト比

    private float viewlength0 = 5.0f; //視点距離
    private float viewlength = 5.0f; //視点距離
    private float viewingangle0 = 45f; //視野角（画面長手方向に対応する視野角[deg]）
    private float viewingangle = 45f; //視野角（画面長手方向に対応する視野角[deg]）
    private float RotAngle=0f;       //2本指回転量
    private float AngleHor=0f;       //横移動角（2本指）[deg]
    private float AngleVer=0f;       //縦移動角（2本指）[deg]

    private float   angle=0f; //回転角度
    private float rotatatinvelocity=1.f;

    //視点変更テスト変数
    private float alph=0f,beta=0f;

    //光源の座標　x,y,z
    private  float[] LightPos={0f,1.5f,3f,1f};//x,y,z,1
    //光源の環境光
    private float[] LightAmb={.15f,.15f,.15f,1f}; //r,g,b,a
    //光源の乱反射光
    private float[] LightDif={.5f,.5f,.5f,1f}; //r,g,b,a
    //光源の鏡面反射反射光
    private float[] LightSpc={.9f,.9f,.9f,1f}; //r,g,b,a

    //変換マトリックス
    private  float[] pMatrix=new float[16]; //プロジェクション変換マトリックス
    private  float[] mMatrix=new float[16]; //モデル変換マトリックス
    private  float[] cMatrix=new float[16]; //カメラビュー変換マトリックス

    //モデル座標系の原点
    private  float[] origin= {0f,0f,0f,1f};

    private Axes MyAxes= new Axes();  //原点周囲の軸表示とためのオブジェクトを作成

    private CircleEX m1CircleEX = new CircleEX(120,32); //半径1の120度の円弧
    private float[] c0 = {0f,1f,1f,1f};
    private float[] c1 = {0f,0f,1f,1f};
    private CircleEX m2CircleEX = new CircleEX(270,64,c0,c1); //半径１の270度のグラド円弧

    private SphereEX m1SphereEX = new SphereEX(360f,-90f,90f,72,36); //半径１の球
    private SphereEX m2SphereEX = new SphereEX(360,0,90,64,16); //半径１の半球
    private TexSphereEX m1TexSphereEX =new TexSphereEX(360f,-90f,90f,72,36); //半径１の球
    private TexSphereEX m2TexSphereEX =new TexSphereEX(360,0,90,64,16); //半径１の半球

    private ConeEX m1ConeEX = new ConeEX(1f,1f,360f,64); //円柱　底面はない
    private ConeEX m2ConeEX = new ConeEX(1f,.5f,360f,64); //円錐台　底面はない
    private ConeEX m3ConeEX = new ConeEX(1f,0f,360f,64); //円錐　底面はない
    private ConeEX m4ConeEX = new ConeEX(0f,.5f,360f,64); //平面リング
    private ConeEX m5ConeEX = new ConeEX(0f,0f,360f,64); //円板
    private ConeEX m6ConeEX = new ConeEX(0f,0f,180f,64); //半円板

    private TexConeEX m1TexConeEX = new TexConeEX(1f,1f,360f,64); //円柱　底面はない
    private TexConeEX m2TexConeEX = new TexConeEX(1f,.5f,360f,64); //円錐台　底面はない
    private TexConeEX m3TexConeEX = new TexConeEX(1f,0f,360f,64); //円錐　底面はない
    private TexConeEX m4TexConeEX = new TexConeEX(0f,.5f,360f,64); //平面リング
    private TexConeEX m5TexConeEX = new TexConeEX(0f,0f,360f,64); //円板
    private TexConeEX m6TexConeEX = new TexConeEX(0f,0f,180f,64); //半円板

    private TorusEX m1TorusEX = new TorusEX(.5f, 360f,0f,360f,72,36); //トーラスを定義
    private TorusEX m2TorusEX = new TorusEX(.5f, 180f,0f,360f,72,36); //半周トーラスを定義
    private TorusEX m3TorusEX = new TorusEX(.5f, 360f,0f,180f,72,36); //上半分トーラスを定義

    private TexTorusEX m1TexTorusEX = new TexTorusEX(.5f, 360f,0f,360f,72,36); //トーラスを定義
    private TexTorusEX m2TexTorusEX = new TexTorusEX(.5f, 180f,0f,360f,72,36); //半周トーラスを定義
    private TexTorusEX m3TexTorusEX = new TexTorusEX(.5f, 360f,0f,180f,72,36); //上半分トーラスを定義

    private Texture EarthPicture;

    GLRenderer(final Context context) {
        mContext = context;
    }

    //サーフェイス生成時に呼ばれる
    @Override
    public void onSurfaceCreated(GL10 gl10, EGLConfig eglConfig) {
        //プログラムの生成
        validProgram = GLES.makeProgram();

        //デプスバッファの有効化
        GLES20.glEnable(GLES20.GL_DEPTH_TEST);

        // カリングの有効化
        GLES20.glEnable(GLES20.GL_CULL_FACE); //裏面を表示しないチェックを行う

        // 裏面を描画しない
        GLES20.glFrontFace(GLES20.GL_CCW); //表面のvertexのindex番号はCCWで登録
        GLES20.glCullFace(GLES20.GL_BACK); //裏面は表示しない

        //背景色の設定
        GLES20.glClearColor(0f, 0f, 0.2f, 1.0f);

        //光源の設定
        GLES.putLightAttribute(LightAmb, LightDif, LightSpc);

        // 背景とのブレンド方法を設定します。
        GLES20.glEnable(GLES20.GL_BLEND);
        GLES20.glBlendFunc(GLES20.GL_SRC_ALPHA, GLES20.GL_ONE_MINUS_SRC_ALPHA);    // 単純なアルファブレンド

        EarthPicture = new Texture(mContext,R.drawable.earthpicture); //テクスチャを作成
    }

    //画面サイズ変更時に呼ばれる
    @Override
    public void onSurfaceChanged(GL10 gl10, int w, int h) {
        //ビューポート変換
        GLES20.glViewport(0,0,w,h);
        aspect=(float)w/(float)h;
    }

    //毎フレーム描画時に呼ばれる
    @Override
    public void onDrawFrame(GL10 glUnused) {
        if (!validProgram) return;

        //画面のクリア
        GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT |
                GLES20.GL_DEPTH_BUFFER_BIT);

        //プロジェクション変換（射影変換）--------------------------------------
        //透視変換（遠近感を作る）
        //カメラは原点に有り，z軸の負の方向を向いていて，上方向はy軸＋方向である。
        GLES.gluPerspective(pMatrix,
                viewingangle,  //Y方向の画角
                aspect, //アスペクト比
                //1.0f,   //ニアクリップ　　　z=-1から
                0.00001f,   //ニアクリップ　　　z=-0.00001から
                100.0f);//ファークリップ　　Z=-100までの範囲を表示することになる
        GLES.setPMatrix(pMatrix);

        //カメラビュー変換（視野変換）-----------------------------------
        //カメラ視点が原点になるような変換
        float[] c1Matrix=new float[16]; //カメラ視点変換マトリックス作成用
        float[] c2Matrix=new float[16]; //カメラ視点変換マトリックス作成用
        Matrix.setLookAtM(c1Matrix, 0,
                (float) (viewlength * Math.sin(beta) * Math.cos(alph)),  //カメラの視点 x
                (float) (viewlength * Math.sin(alph)),                    //カメラの視点 y
                (float) (viewlength * Math.cos(beta) * Math.cos(alph)),  //カメラの視点 z
                0.0f, 0.0f, 0.0f, //カメラの視線方向の代表点
                0.0f, 1.0f, 0.0f);//カメラの上方向
        Matrix.setIdentityM(c2Matrix, 0);
        Matrix.rotateM(c2Matrix, 0, -AngleHor, 0, 1, 0);
        Matrix.rotateM(c2Matrix, 0, -AngleVer, 1, 0, 0);
        Matrix.rotateM(c2Matrix, 0, -RotAngle, 0, 0, 1);
        Matrix.multiplyMM(cMatrix, 0, c2Matrix, 0, c1Matrix, 0); //cMatrix = c2Matrix * c1Matrix
        //カメラ視点ビュー変換はこれで終わり。
        GLES.setCMatrix(cMatrix);

        //cMatrixをセットしてから光源位置をセット
        GLES.setLightPosition(LightPos);

        GLES.selectProgram(GLES.SP_SimpleObject);

        //座標軸の描画
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        //座標軸の描画本体
        MyAxes.draw();//座標軸の描画本体

        //円弧を描く
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.translateM(mMatrix,0,0f,1f,0f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        m1CircleEX.draw(1f,05f,.0f,1f, 1.5f);

        GLES.selectProgram(GLES.SP_SimpleObjectVTXColor);

        //グラデュエーション円弧を描く
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.translateM(mMatrix,0,0f,.9f,0f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        m2CircleEX.drawGradation(2f);

        GLES.selectProgram(GLES.SP_ObjectWithLight);

        //球を描く
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.translateM(mMatrix,0,.6f,.6f,0f);
        Matrix.scaleM(mMatrix,0,.2f,.2f,.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        m1SphereEX.draw(.5f,.5f,.1f,1f,5f);

        //部分球を描く
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.translateM(mMatrix,0,.6f,0f,0f);
        Matrix.scaleM(mMatrix,0,.2f,.2f,.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        m2SphereEX.draw(.5f,.5f,.1f,1f,5f);

        GLES.selectProgram(GLES.SP_TextureWithLight);

        //テクスチャ付きの球を描く
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.translateM(mMatrix,0,.6f,-.4f,0f);
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.scaleM(mMatrix,0,.2f,.2f,.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        EarthPicture.setTexture();
        m1TexSphereEX.draw(1f,1f,1f,1f,5f);

        //テクスチャ付きの半球を描く
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.translateM(mMatrix,0,.6f,-1f,0f);
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.scaleM(mMatrix,0,.2f,.2f,.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        EarthPicture.setTexture();
        m2TexSphereEX.draw(1f,1f,1f,1f,5f);

        GLES.selectProgram(GLES.SP_ObjectWithLight);

        //円柱を描く
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.translateM(mMatrix,0,0f,-1f,-.6f);
        Matrix.scaleM(mMatrix,0,.2f,.2f,.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        m1ConeEX.draw(1f,1f,1f,1f,5f);

        //円錐台を描く
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.translateM(mMatrix,0,0f,-.5f,-.6f);
        Matrix.scaleM(mMatrix,0,.2f,.2f,.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        m2ConeEX.draw(1f,1f,1f,1f,5f);

        //円錐を描く
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.translateM(mMatrix,0,0f,-.1f,-.6f);
        Matrix.scaleM(mMatrix,0,.2f,.2f,.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        m3ConeEX.draw(1f,1f,1f,1f,5f);

        //平面リングを描く
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.translateM(mMatrix,0,0f,.2f,-.6f);
        Matrix.scaleM(mMatrix,0,.2f,.2f,.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        m4ConeEX.draw(1f,1f,1f,1f,5f);

        //円板を描く
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.translateM(mMatrix,0,0f,.5f,-.6f);
        Matrix.scaleM(mMatrix,0,.2f,.2f,.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        m5ConeEX.draw(1f,1f,1f,1f,5f);

        //半円板を描く
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.translateM(mMatrix,0,0f,.8f,-.6f);
        Matrix.scaleM(mMatrix,0,.2f,.2f,.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        m6ConeEX.draw(1f,1f,1f,1f,5f);


        GLES.selectProgram(GLES.SP_TextureWithLight);

        //テクスチャ付きの円柱を描く
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.translateM(mMatrix,0,0f,-1f,.6f);
        Matrix.scaleM(mMatrix,0,.2f,.2f,.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        EarthPicture.setTexture();
        m1TexConeEX.draw(1f,1f,1f,1f,5f);

        //テクスチャ付きの円錐台を描く
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.translateM(mMatrix,0,0f,-.5f,.6f);
        Matrix.scaleM(mMatrix,0,.2f,.2f,.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        EarthPicture.setTexture();
        m2TexConeEX.draw(1f,1f,1f,1f,5f);

        //テクスチャ付きの円錐を描く
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.translateM(mMatrix,0,0f,-.1f,.6f);
        Matrix.scaleM(mMatrix,0,.2f,.2f,.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        EarthPicture.setTexture();
        m3TexConeEX.draw(1f,1f,1f,1f,5f);

        //テクスチャ付きの平面リングを描く
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.translateM(mMatrix,0,0f,.2f,.6f);
        Matrix.scaleM(mMatrix,0,.2f,.2f,.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        EarthPicture.setTexture();
        m4TexConeEX.draw(1f,1f,1f,1f,5f);

        //テクスチャ付きの円板を描く
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.translateM(mMatrix,0,0f,.5f,.6f);
        Matrix.scaleM(mMatrix,0,.2f,.2f,.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        EarthPicture.setTexture();
        m5TexConeEX.draw(1f,1f,1f,1f,5f);

        //テクスチャ付きの半円板を描く
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.translateM(mMatrix,0,0f,.8f,.6f);
        Matrix.scaleM(mMatrix,0,.2f,.2f,.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        EarthPicture.setTexture();
        m6TexConeEX.draw(1f,1f,1f,1f,5f);

        GLES.selectProgram(GLES.SP_ObjectWithLight);

        //トーラスを描く
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.translateM(mMatrix,0,-.6f,-1f,0f);
        Matrix.scaleM(mMatrix,0,.2f,.2f,.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        m1TorusEX.draw(1f,1f,1f,1f,5f);

        //部分トーラスを描く
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.translateM(mMatrix,0,-.6f,-.6f,0f);
        Matrix.scaleM(mMatrix,0,.2f,.2f,.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        m2TorusEX.draw(1f,1f,1f,1f,5f);

        //部分トーラスを描く
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.translateM(mMatrix,0,-.6f,-.2f,0f);
        Matrix.scaleM(mMatrix,0,.2f,.2f,.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        m3TorusEX.draw(1f,1f,1f,1f,5f);

        GLES.selectProgram(GLES.SP_TextureWithLight);

        //テクスチャ付きのトーラスを描く
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.translateM(mMatrix,0,-.6f,.2f,0f);
        Matrix.scaleM(mMatrix,0,.2f,.2f,.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        EarthPicture.setTexture();
        m1TexTorusEX.draw(1f,1f,1f,1f,5f);

        //テクスチャ付きの部分トーラスを描く
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.translateM(mMatrix,0,-.6f,.6f,0f);
        Matrix.scaleM(mMatrix,0,.2f,.2f,.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        EarthPicture.setTexture();
        m2TexTorusEX.draw(1f,1f,1f,1f,5f);

        //テクスチャ付きの部分トーラスを描く
        Matrix.setIdentityM(mMatrix, 0);//モデル変換行列mMatrixを単位行列にする。
        Matrix.rotateM(mMatrix,0,0.5f*angle,0f,1f,0f);
        Matrix.translateM(mMatrix,0,-.6f,1f,0f);
        Matrix.scaleM(mMatrix,0,.2f,.2f,.2f);
        GLES.updateMatrix(mMatrix);//現在の変換行列をシェーダに指定
        EarthPicture.setTexture();
        m3TexTorusEX.draw(1f,1f,1f,1f,5f);

        angle+=0.5f*rotatatinvelocity;

    }

    public void setRotationVelocity(float rotv) {
        this.rotatatinvelocity=rotv;
    }
    public void setTextview(TextView tv) {
        mTextViewTime=tv;
    }

    //一本指でのスクロール
    private float Scroll[] = {0f, 0f}; //１本指のドラッグ[rad]
    public void setScrollValue(float DeltaX, float DeltaY) {
        Scroll[0] -= DeltaX * 0.01;
        if (3.14f < Scroll[0]) Scroll[0] = 3.14f;
        if (Scroll[0] < -3.14) Scroll[0] = -3.14f;
        Scroll[1] += DeltaY * 0.01;
        if (1.57f < Scroll[1]) Scroll[1] = 1.57f;
        if (Scroll[1] < -1.57) Scroll[1] = -1.57f;
        alph = Scroll[1];
        beta = Scroll[0];
    }


    //２本指でのスクロール
    private float Scroll2fg[] = {0f, 0f}; //２本指のドラッグ[deg]
    public void setScroll2Value(float Delta2X, float Delta2Y) {
        Scroll2fg[0] += Delta2X * 0.05; //[deg]
        Scroll2fg[1] += Delta2Y * 0.05; //[deg]
        AngleHor = Scroll2fg[0];
        AngleVer = Scroll2fg[1];
    }

    public void setScroll3Value(float Delta2X, float Delta2Y) {

    }

    //２本指でのピンチイン・ピンチアウト
    private float My_ScaleFactor = 1.0f;
    public void setPinch2(float factor) {
        My_ScaleFactor *= factor;
        viewlength = viewlength0 / My_ScaleFactor;
    }

    //３本指でのピンチイン・ピンチアウト
    private float My_ScaleFactor3 = 1.0f;
    public void setPinch3(float factor) {
        My_ScaleFactor3 *= factor;
        viewingangle = viewingangle0 / My_ScaleFactor3;
    }

    //２本指でのローテート
    private float My_RotAngle =0f;
    public void setRot2Angle(float Rotation) {
        My_RotAngle += Rotation;
        RotAngle = My_RotAngle;
    }

    public void setRot3Angle(float Rotation) {

    }

}
