package jp.gr.java_conf.coskx.gles2sample14p;

import android.opengl.GLES20;

import java.nio.FloatBuffer;

/**
 * Created by tommy on 2016/10/15.
 * ConeEXをテクスチャが貼れるように拡張したクラスです
 */

public class TexConeEX extends ConeEX {
    //bufferの定義
    private FloatBuffer texcoordBuffer;

    TexConeEX(){}
    TexConeEX(float height, float radius2, float maxlongitude, int nSlices) {
        makeCone(height, radius2, maxlongitude, nSlices);
        makeTexCone(height, radius2, maxlongitude, nSlices);
    }

    private void makeTexCone(float height, float radius2, float maxlongitude, int nSlices) {
        int sizeArray = nPoints * 2;
        float textcoords[] = new float[sizeArray];
        float dx = 1f / (float) nSlices;
        int p = 0;
        for (int i = 0; i <= nSlices; i++) {
            for (int j = 0; j <= 1; j++) {
                textcoords[p++] = i * dx;        // x
                textcoords[p++] = 1f - j;        // y
            }
        }
        texcoordBuffer = BufferUtil.makeFloatBuffer(textcoords);
    }
    /************************************
     nSlices = 5 のとき
     1 3 5 7 9 11
     0 2 4 6 8 10
     texture座標系
     (0,0)               (1,0) →x

     (0,1)               (1,1)
     ↓y
     *************************************/

    public void draw(float r,float g,float b,float a, float shininess){
        //頂点点列のテクスチャ座標
        GLES20.glVertexAttribPointer(GLES.texcoordHandle, 2,
                GLES20.GL_FLOAT, false, 0, texcoordBuffer);

        //頂点点列
        GLES20.glVertexAttribPointer(GLES.positionHandle, 3,
                GLES20.GL_FLOAT, false, 0, vertexBuffer);

        if (GLES.checkLiting()) {

            //頂点での法線ベクトル
            GLES20.glVertexAttribPointer(GLES.normalHandle, 3,
                    GLES20.GL_FLOAT, false, 0, normalBuffer);

            //周辺光反射
            GLES20.glUniform4f(GLES.materialAmbientHandle, r, g, b, a);

            //拡散反射
            GLES20.glUniform4f(GLES.materialDiffuseHandle, r, g, b, a);

            //鏡面反射
            GLES20.glUniform4f(GLES.materialSpecularHandle, 1f, 1f, 1f, a);
            GLES20.glUniform1f(GLES.materialShininessHandle, shininess);

        }

        //描画
        indexBuffer.position(0);
        GLES20.glDrawElements(GLES20.GL_TRIANGLE_STRIP,
                nIndexs, GLES20.GL_UNSIGNED_SHORT, indexBuffer);

    }
}
