package jp.gr.java_conf.coskx.stereoprojtest;

import android.opengl.GLES20;
import android.opengl.Matrix;
import android.util.Log;

/**
 * Created by tommy on 2015/06/18.
 */
//シェーダ操作
public class GLES {

    //頂点シェーダのコード
    private final static String VERTEX_CODE=
        //shadingを使用するflag 1の時使用する，0の時使用しない（単色にする）
        "uniform int u_EnableShading;"+
        //shadingを使用しない時の色の設定（単色）
        "uniform vec4 u_ObjectColor;"+

        //光源
        "uniform vec4 u_LightAmbient;"+ //光源の環境光色
        "uniform vec4 u_LightDiffuse;"+ //光源の拡散光色
        "uniform vec4 u_LightSpecular;"+//光源の鏡面光色
        "uniform vec4 u_LightPos;"+     //光源の位置（カメラビュー座標系）

        //マテリアル
        "uniform vec4 u_MaterialAmbient;"+   //マテリアルの環境光色
        "uniform vec4 u_MaterialDiffuse;"+   //マテリアルの拡散光色
        "uniform vec4 u_MaterialSpecular;"+  //マテリアルの鏡面光色
        "uniform float u_MaterialShininess;"+//マテリアルの鏡面指数

        //行列
        "uniform mat4 u_MMatrix;" +       //モデルビュー行列
        "uniform mat4 u_PMMatrix;" +       //射影行列×モデルビュー行列

        //頂点情報
        "attribute vec4 a_Position;"+  //位置
        "attribute vec3 a_Normal;"+     //法線ベクトル

        //出力
        "varying vec4 v_Color;"+ "" +

        //stereographic projection
        "uniform float u_aspect;"+  //portrait:aspect<1,landscae:1<aspect
        "uniform float u_const1;"+ //1/(tan (α/2))
        "uniform float u_const2;"+ //tan(α)
        "uniform int u_projection;"+ //0:perspective projection  1:stereographic projection

        "float atan2(in float y, in float x){"+
        "   return x == 0.0 ? sign(y)*3.1415926535/2. : atan(y, x);"+
        "}"+

		"vec4 stereoprojection(vec4 pos) {"+
			"vec4 a_Position2=u_PMMatrix*pos;"+
			"if (0.0000001<a_Position2.w) {"+
				//"float aspect=10./17.;"+
				"float c1=u_const1*a_Position2.w;"+ //a/(tan (α/2)) a=1*a_Position2.w
				"float c2=u_const2/a_Position2.w;"+ //tan(α)/a      a=1*a_Position2.w
				"if (u_aspect<1.) {"+
					"a_Position2.x *=u_aspect;"+
					"float thetap=atan2(a_Position2.y,a_Position2.x);"+
					"float xprj=sqrt(a_Position2.x*a_Position2.x+a_Position2.y*a_Position2.y);"+
					"float yprj=c1*tan(atan(c2*xprj)/2.);"+
					"a_Position2.x=yprj*cos(thetap);"+
					"a_Position2.y=yprj*sin(thetap);"+
					"a_Position2.x /=u_aspect;"+
				"} else {"+
					"a_Position2.y /=u_aspect;"+
					"float thetap=atan2(a_Position2.y,a_Position2.x);"+
					"float xprj=sqrt(a_Position2.x*a_Position2.x+a_Position2.y*a_Position2.y);"+
					"float yprj=c1*tan(atan(c2*xprj)/2.);"+
					"a_Position2.x=yprj*cos(thetap);"+
					"a_Position2.y=yprj*sin(thetap);"+
					"a_Position2.y *=u_aspect;"+
				"}"+
			"}"+
			"return a_Position2;"+
		"}"+

        "void main(){"+
            "if (u_EnableShading==1) {"+
                //環境光の計算
                "vec4 ambient=u_LightAmbient*u_MaterialAmbient;"+

                //拡散光の計算
                "vec3 P=vec3(u_MMatrix*a_Position);"+
                "vec3 L=normalize(vec3(u_LightPos)-P);"+  //光源方向単位ベクトル
                "vec3 N=normalize(mat3(u_MMatrix)*a_Normal);"+  //法線単位ベクトル
                "float dotLN=max(dot(L,N),0.0);"+
                "vec4 diffuseP=vec4(dotLN);"+
                "vec4 diffuse=diffuseP*u_LightDiffuse*u_MaterialDiffuse;"+

                //鏡面光の計算
                "vec3 V=normalize(-P);"+  //視点方向単位ベクトル
                "float dotNLEffect=ceil(dotLN);"+
                "vec3 R=2.*dotLN*N-L;"+
                "float specularP=pow(max(dot(R,V),0.0),u_MaterialShininess)*dotNLEffect;"+

                "vec4 specular=specularP*u_LightSpecular*u_MaterialSpecular;"+

                //色の指定
                "v_Color=ambient+diffuse+specular;"+
            "} else {"+
                "v_Color=u_ObjectColor;"+
            "}"+

            //位置の指定
            "if (u_projection==1) {"+ //stereographic projection
                /*
                "vec4 a_Position2=u_PMMatrix*a_Position;"+
                //"float aspect=10./17.;"+
                "float c1=u_const1*a_Position2.w;"+ //a/(tan (α/2)) a=1*a_Position2.w
                "float c2=u_const2/a_Position2.w;"+ //tan(α)/a      a=1*a_Position2.w
                //"float c1=5.0;"+
                //"float c2=0.41;"+
                //"a_Position2.x/=a_Position2.w;"+
                //"a_Position2.y/=a_Position2.w;"+
                "if (u_aspect<1.) {"+
                "a_Position2.x *=u_aspect;"+
                "} else {"+
                "a_Position2.y /=u_aspect;"+
                "}"+
                "float thetap=atan2(a_Position2.y,a_Position2.x);"+
                "float xprj=sqrt(a_Position2.x*a_Position2.x+a_Position2.y*a_Position2.y);"+
                "float yprj=c1*tan(atan(c2*xprj)/2.);"+
                "a_Position2.x=yprj*cos(thetap);"+
                "a_Position2.y=yprj*sin(thetap);"+
                "if (u_aspect<1.) {"+
                "a_Position2.x /=u_aspect;"+
                "} else {"+
                "a_Position2.y *=u_aspect;"+
                "}"+
                //"a_Position2.x*=a_Position2.w;"+
                //"a_Position2.y*=a_Position2.w;"+
                "gl_Position = a_Position2;"+
                */
                "gl_Position = stereoprojection(a_Position);"+
            "} else {"+ //perspective projection
                "gl_Position = u_PMMatrix*a_Position;"+
            "}"+
        "}";

    //フラグメントシェーダのコード
    private final static String FRAGMENT_CODE=
        "precision mediump float;"+
        "varying vec4 v_Color;"+
        "void main(){"+
            "gl_FragColor=v_Color;"+
        "}";

    //システム
    public static int enableShadingHandle; //shadingを行うflagのハンドル
    public static int objectColorHandle;   //shadingを行わない時に使う単色ハンドル

    //光源のハンドル
    public static int lightAmbientHandle; //光源の環境光色ハンドル
    public static int lightDiffuseHandle; //光源の拡散光色ハンドル
    public static int lightSpecularHandle;//光源の鏡面光色ハンドル
    public static int lightPosHandle;     //光源の位置ハンドル


    //マテリアルのハンドル
    public static int materialAmbientHandle;  //マテリアルの環境光色ハンドル
    public static int materialDiffuseHandle;  //マテリアルの拡散光色ハンドル
    public static int materialSpecularHandle; //マテリアルの鏡面光色ハンドル
    public static int materialShininessHandle;//マテリアルの鏡面指数ハンドル

    //行列のハンドル
    public static int mMatrixHandle;     //モデルビュー行列ハンドル（カメラビュー行列×モデル変換行列）
    public static int pmMatrixHandle;     //(射影行列×モデルビュー行列)ハンドル

    //頂点のハンドル
    public static int positionHandle;//位置ハンドル
    public static int normalHandle;  //法線ハンドル

    //行列
    public static float[] cMatrix=new float[16];//カメラビュー変換行列
    public static float[] mvMatrix=new float[16];//モデルビュー変換行列
    public static float[] pMatrix=new float[16];//プロジェクション行列（射影行列）
    public static float[] pmMatrix=new float[16];//pMatrix*mvMatrix

    //光源
    private static float[] LightPos = new float[4];    //光源の座標　x,y,z　（ワールド座標）
    private static float[] CVLightPos = new float[4];  //光源の座標　x,y,z　（カメラビュー座標）

    //projection関係のハンドル 変数
    public static int aspectHandle;
    public static int const1Handle;
    public static int const2Handle;
    public static int projectionHandle;
    private static float s_aspect; //w/h portrait:aspect<1,landscae:1<aspect
    private static double s_viewangle; //[rad]
    private static int s_projection; //0:perspective projection  1:stereographic projection

    //プログラムの生成
    public static boolean makeProgram() {
        int myProgram;//プログラムオブジェクト
        //シェーダーオブジェクトの生成
        int vertexShader=loadShader(GLES20.GL_VERTEX_SHADER,VERTEX_CODE);
        if (vertexShader==-1) return false;
        int fragmentShader=loadShader(GLES20.GL_FRAGMENT_SHADER,FRAGMENT_CODE);
        if (fragmentShader==-1) return false;

        //プログラムオブジェクトの生成
        myProgram=GLES20.glCreateProgram();
        GLES20.glAttachShader(myProgram, vertexShader);
        GLES20.glAttachShader(myProgram, fragmentShader);
        GLES20.glLinkProgram(myProgram);

        // リンクエラーチェック
        int[] linked = new int[1];
        GLES20.glGetProgramiv(myProgram, GLES20.GL_LINK_STATUS, linked, 0);
        if (linked[0] <= 0) {
            Log.e(" makeProgram", "Failed in Linking");
            Log.e(" makeProgram", GLES20.glGetProgramInfoLog(myProgram));
            return false;
        }

        //shading可否ハンドルの取得
        enableShadingHandle=GLES20.glGetUniformLocation(myProgram,"u_EnableShading");

        //光源のハンドルの取得
        lightAmbientHandle=GLES20.glGetUniformLocation(myProgram,"u_LightAmbient");
        lightDiffuseHandle=GLES20.glGetUniformLocation(myProgram,"u_LightDiffuse");
        lightSpecularHandle=GLES20.glGetUniformLocation(myProgram,"u_LightSpecular");
        lightPosHandle=GLES20.glGetUniformLocation(myProgram,"u_LightPos");

        //マテリアルのハンドルの取得
        materialAmbientHandle=GLES20.glGetUniformLocation(myProgram,"u_MaterialAmbient");
        materialDiffuseHandle=GLES20.glGetUniformLocation(myProgram,"u_MaterialDiffuse");
        materialSpecularHandle=GLES20.glGetUniformLocation(myProgram,"u_MaterialSpecular");
        materialShininessHandle=GLES20.glGetUniformLocation(myProgram,"u_MaterialShininess");
        //光源を使わない時のマテリアルの色のハンドルの取得
        objectColorHandle=GLES20.glGetUniformLocation(myProgram,"u_ObjectColor");

        //行列のハンドルの取得
        mMatrixHandle=GLES20.glGetUniformLocation(myProgram,"u_MMatrix");
        pmMatrixHandle = GLES20.glGetUniformLocation(myProgram, "u_PMMatrix");

        //頂点とその法線ベクトルのハンドルの取得
        positionHandle=GLES20.glGetAttribLocation(myProgram, "a_Position");
        normalHandle=GLES20.glGetAttribLocation(myProgram, "a_Normal");

        //projection関係のハンドルの取得
        projectionHandle=GLES20.glGetUniformLocation(myProgram,"u_projection");
        aspectHandle=GLES20.glGetUniformLocation(myProgram,"u_aspect");
        const1Handle=GLES20.glGetUniformLocation(myProgram,"u_const1");
        const2Handle=GLES20.glGetUniformLocation(myProgram,"u_const2");

        //プログラムオブジェクトの利用開始
        GLES20.glUseProgram(myProgram);
        enableShading();

        return true;
    }

    //シェーダーオブジェクトの生成
    private static int loadShader(int type,String shaderCode) {
        int shader=GLES20.glCreateShader(type);
        GLES20.glShaderSource(shader, shaderCode);
        GLES20.glCompileShader(shader);
        // コンパイルチェック
        int[] compiled = new int[1];
        GLES20.glGetShaderiv(shader, GLES20.GL_COMPILE_STATUS, compiled, 0);
        if (compiled[0] == 0) {
            Log.e(" loadShader", "Failed in Compilation");
            Log.e(" loadShader", GLES20.glGetShaderInfoLog(shader));
            return -1;
        }
        return shader;
    }

    //透視変換の指定
    //angle   [deg]長手方向の見込む角度[deg]
    //aspect  w/h portrait:aspect<1,landscae:1<aspect
    public static void gluPerspective(float[] pm, float angle, float aspect, float near, float far) {
        //portraitのときw<h, aspect<1
        //landscapeのときw>h, 1<aspect
        float top,bottom,left,right;
        if(aspect<1f) {//portrait
            top = near * (float) Math.tan(angle * (Math.PI / 360.0));
            bottom = -top;
            left = bottom * aspect;
            right = -left;
        } else {//landscape
            right = 1.1f*near * (float) Math.tan(angle * (Math.PI / 360.0));
            left = -right;
            bottom = left / aspect;
            top = -bottom;
        }
        Matrix.frustumM(pm, 0, left, right, bottom, top, near, far);
        s_aspect=aspect;
        s_viewangle=angle*Math.PI/180.;
    }

    //ワールド座標系のLightPosを受け取る
    public static void setLightPosition(float[] lp) {
        System.arraycopy(lp, 0, LightPos, 0, 4);
    }

    //プロジェクション行列（射影行列）を受け取る
    public static void setPMatrix(float[] pm) {
        System.arraycopy(pm, 0, pMatrix, 0, 16);
    }

    //カメラビュー変換行列を受け取る
    public static void setCMatrix(float[] cm) {
        System.arraycopy(cm, 0, cMatrix, 0, 16);
    }

    //カメラビュー変換行列×モデル変換行列 = モデルビュー行列をシェーダに指定
    public static void updateMatrix(float[] mm) {
        Matrix.multiplyMM(mvMatrix, 0, cMatrix, 0, mm, 0);       //mvMatrix = cMatrix * mm
        Matrix.multiplyMM(pmMatrix, 0, pMatrix, 0, mvMatrix, 0); //pmMatrix = pMatrix * mvMatrix
        //モデルビュー行列をシェーダに指定
        GLES20.glUniformMatrix4fv(mMatrixHandle, 1, false, mvMatrix, 0);

        //プロジェクション行列（射影行列）×モデルビュー行列をシェーダに指定
        GLES20.glUniformMatrix4fv(pmMatrixHandle, 1, false, pmMatrix, 0);

        //シェーダはカメラビュー座標系の光源位置を使う
        //ワールド座標系のLightPosを，カメラビュー座標系に変換してシェーダに送る
        Matrix.multiplyMV(CVLightPos, 0, cMatrix, 0, LightPos, 0);
        GLES20.glUniform4f(GLES.lightPosHandle, CVLightPos[0], CVLightPos[1], CVLightPos[2], 1.0f);

        GLES20.glUniform1i(projectionHandle,s_projection);
        GLES20.glUniform1f(aspectHandle,s_aspect);
        double alpha=s_viewangle/2.;
        GLES20.glUniform1f(const1Handle,(float)(1/Math.tan(alpha/2.))); //1/tan(α/2)
        GLES20.glUniform1f(const2Handle,(float)Math.tan(alpha));        //tan(α)

    }

    public static void transformPCM(float[] result, float[] source ) {
        Matrix.multiplyMV(result, 0, GLES.pmMatrix, 0, source, 0);
        result[0]/=result[3];
        result[1]/=result[3];
        result[2]/=result[3];
        //result[3]にはsourceのz要素が符号反転されて入っている
    }

    public static void enableShading() {
        GLES20.glUniform1i(enableShadingHandle, 1);
    }
    public static void disableShading() {
        GLES20.glUniform1i(enableShadingHandle,0);
    }

    //projection  0:perspective projection  1:stereographic projection
    public static void setProjectionMethod(int projection) {
        s_projection=projection;
    }

}
